@@ -4,7 +4,7 @@
 #include <unistd.h>
 #include "owl.h"
 
-static const char fileIdent[] = "$Id: commands.c,v 1.53 2003/10/03 23:34:01 kretch Exp $";
+static const char fileIdent[] = "$Id: commands.c,v 1.76 2004/04/10 14:47:41 kretch Exp $";
 
 /* fn is "char *foo(int argc, char **argv, char *buff)" */
 #define OWLCMD_ARGS(name, fn, ctx, summary, usage, description) \
@@ -134,6 +134,11 @@ owl_cmd commands_to_init[]
 	      "aimzwrite <user>",
 	      "Send an aim message to a user.\n"),
 
+  OWLCMD_ARGS("loopwrite", owl_command_loopwrite, OWL_CTX_INTERACTIVE,
+	      "send a loopback message",
+	      "loopwrite",
+	      "Send a local message.\n"),
+
   OWLCMD_ARGS("zcrypt", owl_command_zcrypt, OWL_CTX_INTERACTIVE,
 	      "send an encrypted zephyr",
 	      "zcrypt [-n] [-C] [-c class] [-i instance] [-r realm] [-O opcde] [-m <message...>]\n",
@@ -153,12 +158,13 @@ owl_cmd commands_to_init[]
 
   OWLCMD_ARGS("set", owl_command_set, OWL_CTX_ANY,
 	      "set a variable value",
-	      "set [-q] <variable> [<value>]\n"
+	      "set [-q] [<variable>] [<value>]\n"
 	      "set",
 	      "Set the named variable to the specified value.  If no\n"
-	      "arguments are used print the value of all variables.\n"
-	      "If value is unspecified and the variable is a boolean, will set it to 'on'.\n"
-	      "If -q is specified, is silent and doesn't print a message.\n"),
+	      "arguments are given, print the value of all variables.\n"
+	      "If the value is unspecified and the variable is a boolean, it will be\n"
+	      "set to 'on'.  If -q is used, set is silent and does not print a\n"
+	      "message.\n"),
 
   OWLCMD_ARGS("unset", owl_command_unset, OWL_CTX_ANY,
 	      "unset a boolean variable value",
@@ -231,6 +237,11 @@ owl_cmd commands_to_init[]
 	      "source <filename>",
 	      "Execute the owl commands in <filename>.\n"),
 
+  OWLCMD_ARGS("aim", owl_command_aim, OWL_CTX_INTERACTIVE,
+	      "AIM specific commands",
+	      "aim search <email>",
+	      ""),
+
   OWLCMD_ARGS("addbuddy", owl_command_addbuddy, OWL_CTX_INTERACTIVE,
 	      "add a buddy to a buddylist",
 	      "addbuddy aim <screenname>",
@@ -245,6 +256,11 @@ owl_cmd commands_to_init[]
 	      "such as zephyr, will also be able to use this command.  For now the\n"
 	      "only available protocol is 'aim', specified as the first argument.\n"),
 
+  OWLCMD_ARGS("join", owl_command_join, OWL_CTX_INTERACTIVE,
+	      "join a chat group",
+	      "join aim <groupname> [exchange]",
+	      "Join the AIM chatroom with 'groupname'.\n"),
+
   OWLCMD_ARGS("smartzpunt", owl_command_smartzpunt, OWL_CTX_INTERACTIVE,
 	      "creates a zpunt based on the current message",
 	      "smartzpunt [-i | --instance]",
@@ -288,7 +304,7 @@ owl_cmd commands_to_init[]
 
   OWLCMD_ARGS("blist", owl_command_blist, OWL_CTX_INTERACTIVE,
 	      "List all buddies logged in",
-	      "alist",
+	      "blist",
 	      "Print a listing of buddies logged in, regardless of protocol."),
 
   OWLCMD_ARGS("toggle-oneline", owl_command_toggleoneline, OWL_CTX_INTERACTIVE,
@@ -434,11 +450,33 @@ owl_cmd commands_to_init[]
 	      "and 'galeon' are supported.\n"),
 
   OWLCMD_ARGS("zaway", owl_command_zaway, OWL_CTX_INTERACTIVE,
-	      "running a command from the shell",
+	      "Set, enable or disable zephyr away message",
 	      "zaway [ on | off | toggle ]\n"
 	      "zaway <message>",
-	      "Turn on or off the default zaway message.  If a message is\n"
-	      "specified turn on zaway with that message\n"),
+	      "Turn on or off a zaway message.  If 'message' is\n"
+	      "specified turn on zaway with that message, otherwise\n"
+	      "use the default.\n"),
+
+  OWLCMD_ARGS("aaway", owl_command_aaway, OWL_CTX_INTERACTIVE,
+	      "Set, enable or disable AIM away message",
+	      "aaway [ on | off | toggle ]\n"
+	      "aaway <message>",
+	      "Turn on or off the AIM away message.  If 'message' is\n"
+	      "specified turn on aaway with that message, otherwise\n"
+	      "use the default.\n"),
+
+  OWLCMD_ARGS("away", owl_command_away, OWL_CTX_INTERACTIVE,
+	      "Set, enable or disable both AIM and zephyr away messages",
+	      "away [ on | off | toggle ]\n"
+	      "away <message>",
+	      "Turn on or off the AIM and zephyr away message.  If\n"
+	      "'message' is specified turn them on with that message,\n"
+	      "otherwise use the default.\n"
+	      "\n"
+	      "This command really just runs the 'aaway' and 'zaway'\n"
+	      "commands together\n"
+	      "\n"
+	      "SEE ALSO: aaway, zaway"),
 
   OWLCMD_ARGS("load-subs", owl_command_loadsubs, OWL_CTX_ANY,
 	      "load subscriptions from a file",
@@ -481,9 +519,15 @@ owl_cmd commands_to_init[]
 	      "    opcode     -  zephyr opcode\n"
 	      "    realm      -  zephyr realm\n"
 	      "    body       -  message body\n"
+	      "    hostname   -  hostname of sending host\n"
 	      "    type       -  message type (zephyr, aim, admin)\n"
 	      "    direction  -  either 'in' 'out' or 'none'\n"
-	      "    login      -  either 'login' 'logout' or 'none'\n" 
+	      "    login      -  either 'login' 'logout' or 'none'\n"
+	      "Also you may match on the validity of another filter:\n"
+	      "    filter <filtername>\n"
+	      "Also you may pass the message to a perl function returning 0 or 1,\n"
+	      "where 1 indicates that the function matches the filter:\n"
+	      "    perl <subname>\n"
 	      "Valid operators are:\n"
 	      "    and\n"
 	      "    or\n"
@@ -504,9 +548,17 @@ owl_cmd commands_to_init[]
 	      "of valid colors.\n\n"
 	      "SEE ALSO: 'show colors'\n"),
 
+  OWLCMD_ARGS("colorclass", owl_command_colorclass, OWL_CTX_INTERACTIVE,
+	      "create a filter to color messages of the given class name",
+	      "colorclass <class> <color>",
+	      "A filter will be created to color messages in <class>"
+	      "in <color>.  Use the 'show colors' command for a list\n"
+	      "of valid colors.\n\n"
+	      "SEE ALSO: 'show colors'\n"),
+
   OWLCMD_ARGS("view", owl_command_view, OWL_CTX_INTERACTIVE,
 	      "view messages matching a filter",
-	      "view [<viewname>] [-f <filter> | --home ] [-s <style>]\n"
+	      "view [<viewname>] [-f <filter> | --home | -r ] [-s <style>]\n"
 	      "view <filter>\n"
 	      "view -d <expression>\n"
 	      "view --home",
@@ -568,29 +620,32 @@ owl_cmd commands_to_init[]
 
   OWLCMD_ARGS("show", owl_command_show, OWL_CTX_INTERACTIVE,
 	      "show information",
-	      "show variables\n"
-	      "show variable <variable>\n"
+	      "show colors\n"
+	      "show commands\n"
+	      "show command <command>\n"
+	      "show errors\n"
 	      "show filters\n"
 	      "show filter <filter>\n"
 	      "show keymaps\n"
 	      "show keymap <keymap>\n"
-	      "show commands\n"
-	      "show command <command>\n"
-	      "show subs\n"
-	      "show subscriptions\n"
-	      "show zpunts\n"
+	      "show license\n"
+	      "show startup\n"
+	      "show status\n"
 	      "show styles\n"
-	      "show colors\n"
+	      "show subscriptions / show subs\n"
 	      "show terminal\n"
+	      "show variables\n"
+	      "show variable <variable>\n"
 	      "show version\n"
 	      "show view [<view>]\n"
-	      "show status\n",
+	      "show zpunts\n",
 
 	      "Show colors will display a list of valid colors for the\n"
 	      "     terminal."
 	      "Show filters will list the names of all filters.\n"
 	      "Show filter <filter> will show the definition of a particular\n"
 	      "     filter.\n\n"
+	      "Show startup will display the custom startup config\n\n"
 	      "Show zpunts will show the active zpunt filters.\n\n"
 	      "Show keymaps will list the names of all keymaps.\n"
 	      "Show keymap <keymap> will show the key bindings in a keymap.\n\n"
@@ -599,6 +654,7 @@ owl_cmd commands_to_init[]
 	      "Show styles will list the names of all styles available\n"
 	      "for formatting messages.\n\n"
 	      "Show variables will list the names of all variables.\n\n"
+	      "Show errors will show a list of errors ecountered by Owl.\n\n"
 	      "SEE ALSO: filter, view, alias, bindkey, help\n"),
   
   OWLCMD_ARGS("delete", owl_command_delete, OWL_CTX_INTERACTIVE,
@@ -947,6 +1003,26 @@ void owl_command_version()
   owl_function_makemsg(buff);
 }
 
+char *owl_command_aim(int argc, char **argv, char *buff)
+{
+  if (argc<2) {
+    owl_function_makemsg("not enough arguments to aim command");
+    return(NULL);
+  }
+
+  if (!strcmp(argv[1], "search")) {
+    if (argc!=3) {
+      owl_function_makemsg("not enough arguments to aim search command");
+      return(NULL);
+    }
+    owl_aim_search(argv[2]);
+  } else {
+    owl_function_makemsg("unknown subcommand '%s' for aim command", argv[1]);
+    return(NULL);
+  }
+  return(NULL);
+}
+
 char *owl_command_addbuddy(int argc, char **argv, char *buff)
 {
   if (argc!=3) {
@@ -987,10 +1063,6 @@ char *owl_command_delbuddy(int argc, char **argv, char *buff)
       owl_function_makemsg("delbuddy: You must be logged into aim to use this command.");
       return(NULL);
     }
-    /*
-    owl_function_makemsg("This function is not yet operational.  Stay tuned.");
-    return(NULL);
-    */
     owl_aim_delbuddy(argv[2]);
     owl_function_makemsg("%s deleted as AIM buddy for %s", argv[2], owl_global_get_aim_screenname(&g));
   } else if (!strcasecmp(argv[1], "zephyr")) {
@@ -1000,7 +1072,30 @@ char *owl_command_delbuddy(int argc, char **argv, char *buff)
     owl_function_makemsg("delbuddy: currently the only supported protocols are 'zephyr' and 'aim'");
   }
 
+  return(NULL);
+}
 
+char *owl_command_join(int argc, char **argv, char *buff)
+{
+  if (argc!=3 && argc!=4) {
+    owl_function_makemsg("usage: join <protocol> <buddyname> [exchange]");
+    return(NULL);
+  }
+
+  if (!strcasecmp(argv[1], "aim")) {
+    if (!owl_global_is_aimloggedin(&g)) {
+      owl_function_makemsg("join aim: You must be logged into aim to use this command.");
+      return(NULL);
+    }
+    if (argc==3) {
+      owl_aim_chat_join(argv[2], 4);
+    } else {
+      owl_aim_chat_join(argv[2], atoi(argv[3]));
+    }
+    /* owl_function_makemsg("%s deleted as AIM buddy for %s", argv[2], owl_global_get_aim_screenname(&g)); */
+  } else {
+    owl_function_makemsg("join: currently the only supported protocol is 'aim'");
+  }
   return(NULL);
 }
 
@@ -1047,12 +1142,15 @@ char *owl_command_unstartup(int argc, char **argv, char *buff)
 
 char *owl_command_dump(int argc, char **argv, char *buff)
 {
+  char *filename;
+  
   if (argc!=2) {
     owl_function_makemsg("usage: dump <filename>");
     return(NULL);
   }
-
-  owl_function_dump(argv[1]);
+  filename=owl_util_makepath(argv[1]);
+  owl_function_dump(filename);
+  owl_free(filename);
   return(NULL);
 }
 
@@ -1139,7 +1237,7 @@ char *owl_command_smartnarrow(int argc, char **argv, char *buff)
     owl_function_makemsg("Wrong number of arguments for %s", argv[0]);    
   }
   if (filtname) {
-    owl_function_change_view(filtname);
+    owl_function_change_currentview_filter(filtname);
     owl_free(filtname);
   }
   return NULL;
@@ -1280,6 +1378,75 @@ char *owl_command_zaway(int argc, char **argv, char *buff)
 }
 
 
+char *owl_command_aaway(int argc, char **argv, char *buff)
+{
+  if ((argc==1) ||
+      ((argc==2) && !strcmp(argv[1], "on"))) {
+    owl_global_set_aaway_msg(&g, owl_global_get_aaway_msg_default(&g));
+    owl_function_aaway_on();
+    return NULL;
+  }
+
+  if (argc==2 && !strcmp(argv[1], "off")) {
+    owl_function_aaway_off();
+    return NULL;
+  }
+
+  if (argc==2 && !strcmp(argv[1], "toggle")) {
+    owl_function_aaway_toggle();
+    return NULL;
+  }
+
+  buff = skiptokens(buff, 1);
+  owl_global_set_aaway_msg(&g, buff);
+  owl_function_aaway_on();
+  return NULL;
+}
+
+
+char *owl_command_away(int argc, char **argv, char *buff)
+{
+  if ((argc==1) ||
+      ((argc==2) && !strcmp(argv[1], "on"))) {
+    owl_global_set_aaway_msg(&g, owl_global_get_aaway_msg_default(&g));
+    owl_global_set_zaway_msg(&g, owl_global_get_zaway_msg_default(&g));
+    owl_function_aaway_on();
+    owl_function_zaway_on();
+    owl_function_makemsg("Away messages set.", owl_global_get_aaway_msg_default(&g));
+    return NULL;
+  }
+
+  if (argc==2 && !strcmp(argv[1], "off")) {
+    owl_function_aaway_off();
+    owl_function_zaway_off();
+    return NULL;
+  }
+
+  if (argc==2 && !strcmp(argv[1], "toggle")) {
+    /* if either one is on, turn it off, otherwise toggle both (turn
+     *  them both on)
+     */
+    if (!owl_global_is_zaway(&g) && !owl_global_is_aaway(&g)) {
+      owl_function_aaway_toggle();
+      owl_function_zaway_toggle();
+      owl_function_makemsg("Away messages set.");
+    } else {
+      if (owl_global_is_zaway(&g)) owl_function_zaway_toggle();
+      if (owl_global_is_aaway(&g)) owl_function_aaway_toggle();
+      owl_function_makemsg("Away messages off.");
+    }
+    return NULL;
+  }
+
+  buff = skiptokens(buff, 1);
+  owl_global_set_aaway_msg(&g, buff);
+  owl_global_set_zaway_msg(&g, buff);
+  owl_function_aaway_on();
+  owl_function_zaway_on();
+  owl_function_makemsg("Away messages set.");
+  return NULL;
+}
+
 char *owl_command_set(int argc, char **argv, char *buff)
 {
   char *var, *val;
@@ -1697,6 +1864,12 @@ char *owl_command_aimwrite(int argc, char **argv, char *buff)
   return(NULL);
 }
 
+char *owl_command_loopwrite(int argc, char **argv, char *buff)
+{
+  owl_function_loopwrite_setup();
+  return(NULL);
+}
+
 char *owl_command_zcrypt(int argc, char **argv, char *buff)
 {
 #ifdef OWL_ENABLE_ZCRYPT
@@ -1784,25 +1957,31 @@ char *owl_command_zlocate(int argc, char **argv, char *buff)
   return NULL;
 }
 
+
+/* Backwards compatability has made this kind of complicated:
+ * view [<viewname>] [-f <filter> | -d <expression> | --home | -r ] [-s <style>]
+ * view <filter>
+ * view -d <expression>
+ * view --home
+ */
 char *owl_command_view(int argc, char **argv, char *buff)
 {
-
-  /* Backwards compatability has made this kind of complicated:
-   * view [<viewname>] [-f <filter> | -d <expression> | --home ] [-s <style>]
-   * view <filter>
-   * view -d <expression>
-   * view --home
-   */
-
-  /* First take the 'view --home' case */
-  if (argc == 2 && !strcmp(argv[1], "--home")) {
-    owl_function_change_view(owl_global_get_view_home(&g));
-    return(NULL);
+  /* First take the 'view --home' and 'view -r' cases */
+  if (argc == 2) {
+    if (!strcmp(argv[1], "--home")) {
+      owl_function_change_currentview_filter(owl_global_get_view_home(&g));
+      return(NULL);
+    } else if (!strcmp(argv[1], "-r")) {
+      char *foo;
+      foo=owl_function_create_negative_filter(owl_view_get_filtname(owl_global_get_current_view(&g)));
+      owl_function_change_currentview_filter(foo);
+      return(NULL);
+    }
   }
 
   /* Now look for 'view <filter>' */
   if (argc==2) {
-    owl_function_change_view(argv[1]);
+    owl_function_change_currentview_filter(argv[1]);
     return(NULL);
   }
 
@@ -1818,7 +1997,7 @@ char *owl_command_view(int argc, char **argv, char *buff)
       myargv[i]=argv[i];
     }
     owl_function_create_filter(argc, myargv);
-    owl_function_change_view("owl-dynamic");
+    owl_function_change_currentview_filter("owl-dynamic");
     owl_free(myargv);
     return NULL;
   }
@@ -1830,7 +2009,11 @@ char *owl_command_view(int argc, char **argv, char *buff)
   }
   argc--;
   argv++;
-  if (strcmp(argv[0], "-f") && strcmp(argv[0], "-d") && strcmp(argv[0], "--home") && strcmp(argv[0], "-s")) {
+  if (strcmp(argv[0], "-f") &&
+      strcmp(argv[0], "-d") &&
+      strcmp(argv[0], "--home") &&
+      strcmp(argv[0], "-s") &&
+      strcmp(argv[0], "-r")) {
     if (strcmp(argv[0], "main")) {
       owl_function_makemsg("No view named '%s'", argv[0]);
       return(NULL);
@@ -1844,13 +2027,17 @@ char *owl_command_view(int argc, char **argv, char *buff)
 	owl_function_makemsg("Too few argments to the view command");
 	return(NULL);
       }
-      owl_function_change_view(argv[1]);
+      owl_function_change_currentview_filter(argv[1]);
       argc-=2;
       argv+=2;
     } else if (!strcmp(argv[0], "--home")) {
-      owl_function_change_view(owl_global_get_view_home(&g));
+      owl_function_change_currentview_filter(owl_global_get_view_home(&g));
       argc--;
       argv++;
+    } else if (!strcmp(argv[0], "-r")) {
+      char *foo;
+      foo=owl_function_create_negative_filter(owl_view_get_filtname(owl_global_get_current_view(&g)));
+      owl_function_change_currentview_filter(foo);
     } else if (!strcmp(argv[0], "-s")) {
       if (argc<2) {
 	owl_function_makemsg("Too few argments to the view command");
@@ -1868,7 +2055,6 @@ char *owl_command_view(int argc, char **argv, char *buff)
   return(NULL);
 }
 
-
 char *owl_command_show(int argc, char **argv, char *buff)
 {
   if (argc<2) {
@@ -1921,6 +2107,16 @@ char *owl_command_show(int argc, char **argv, char *buff)
     owl_function_about();
   } else if (!strcmp(argv[1], "status")) {
     owl_function_status();
+  } else if (!strcmp(argv[1], "license")) {
+    owl_function_show_license();
+  } else if (!strcmp(argv[1], "startup")) {
+    char *filename;
+    
+    filename=owl_sprintf("%s/%s", owl_global_get_homedir(&g), OWL_STARTUP_FILE);
+    owl_function_popless_file(filename);
+    owl_free(filename);
+  } else if (!strcmp(argv[1], "errors")) {
+    owl_function_showerrs();
   } else {
     owl_function_makemsg("Unknown subcommand for 'show' command (see 'help show' for allowed args)");
     return NULL;
@@ -1936,7 +2132,7 @@ char *owl_command_viewclass(int argc, char **argv, char *buff)
     return NULL;
   }
   filtname = owl_function_classinstfilt(argv[1], NULL);
-  owl_function_change_view(filtname);
+  owl_function_change_currentview_filter(filtname);
   owl_free(filtname);
   return NULL;
 }
@@ -1949,7 +2145,7 @@ char *owl_command_viewuser(int argc, char **argv, char *buff)
     return NULL;
   }
   filtname=owl_function_zuserfilt(argv[1]);
-  owl_function_change_view(filtname);
+  owl_function_change_currentview_filter(filtname);
   owl_free(filtname);
   return NULL;
 }
@@ -2043,6 +2239,20 @@ char *owl_command_colorview(int argc, char **argv, char *buff)
   return NULL;
 }
 
+char *owl_command_colorclass(int argc, char **argv, char *buff)
+{
+  char *filtname;
+  
+  if (argc!=3) {
+    owl_function_makemsg("Wrong number of arguments to colorclass command");
+    return NULL;
+  }
+
+  filtname=owl_function_classinstfilt(argv[1], NULL);
+  (void) owl_function_color_filter(filtname, argv[2]);
+  return NULL;
+}
+
 char *owl_command_zpunt(int argc, char **argv, char *buff)
 {
   owl_command_zpunt_and_zunpunt(argc, argv, 0);
@@ -2173,7 +2383,7 @@ char *owl_command_aimlogin(int argc, char **argv, char *buff)
 
   /* try to login */
   ret=owl_aim_login(argv[1], argv[2]);
-  if (ret) owl_function_makemsg("Warning: login for %s failed.\n");
+  if (ret) owl_function_makemsg("Warning: login for %s failed.\n", argv[1]);
 
   /* this is a test */
   return(NULL);
