@@ -4,7 +4,7 @@
 #include <string.h>
 #include <ctype.h>
 
-static const char fileIdent[] = "$Id: editwin.c,v 1.12 2003/06/30 22:12:40 kretch Exp $";
+static const char fileIdent[] = "$Id: editwin.c,v 1.13 2003/10/29 00:56:36 kretch Exp $";
 
 #define INCR 5000
 
@@ -175,7 +175,6 @@ void owl_editwin_clear(owl_editwin *e)
   owl_editwin_adjust_for_locktext(e);
 }
 
-
 /* malloc more space for the buffer */
 void _owl_editwin_addspace(owl_editwin *e)
 {
@@ -772,9 +771,12 @@ void owl_editwin_move_to_end(owl_editwin *e)
   owl_editwin_key_right(e);
 
   /* do we need to scroll? */
+  /*
   if (e->buffy-e->topline > e->winlines) {
     e->topline+=e->winlines/2;
   }
+  */
+  owl_editwin_recenter(e);
 }
 
 void owl_editwin_move_to_top(owl_editwin *e)
