@@ -1,7 +1,9 @@
 #include <string.h>
 #include "owl.h"
 
-static const char fileIdent[] = "$Id: filter.c,v 1.12 2003/07/02 22:15:24 kretch Exp $";
+static const char fileIdent[] = "$Id: filter.c,v 1.20 2003/12/26 19:50:08 nygren Exp $";
+
+#define OWL_FILTER_MAXRECURSE 20
 
 int owl_filter_init_fromstring(owl_filter *f, char *name, char *string)
 {
@@ -16,9 +18,10 @@ int owl_filter_init_fromstring(owl_filter *f, char *name, char *string)
 
 int owl_filter_init(owl_filter *f, char *name, int argc, char **argv)
 {
-  int i, error;
+  int i, j, error;
   owl_filterelement *fe;
   char *regexstr;
+  owl_list list;
    
   f->name=owl_strdup(name);
   f->polarity=0;
@@ -29,7 +32,11 @@ int owl_filter_init(owl_filter *f, char *name, int argc, char **argv)
   /* first take arguments that have to come first */
   /* set the color */
   if (argc>=2 && !strcmp(argv[0], "-c")) {
-    f->color=owl_util_string_to_color(argv[1]);
+    if (owl_util_string_to_color(argv[1])==-1) {
+      owl_function_error("The color '%s' is not available, using default.", argv[1]);
+    } else {
+      f->color=owl_util_string_to_color(argv[1]);
+    }
     argc-=2;
     argv+=2;
   }
@@ -55,7 +62,7 @@ int owl_filter_init(owl_filter *f, char *name, int argc, char **argv)
     } else if (!strcasecmp(argv[i], "false")) {
       owl_filterelement_create_false(fe);
       
-    } else if (i==argc-1) {
+    } else if (i==argc-1) { /* we need more than one arg at this point */
       error=1;
     } else {
       if (!strcasecmp(argv[i], "class") ||
@@ -67,11 +74,18 @@ int owl_filter_init(owl_filter *f, char *name, int argc, char **argv)
 	  !strcasecmp(argv[i], "realm") ||
 	  !strcasecmp(argv[i], "type") ||
 	  !strcasecmp(argv[i], "direction") ||
+	  !strcasecmp(argv[i], "hostname") ||
 	  !strcasecmp(argv[i], "login")) {
-	regexstr=owl_util_substitute(argv[i+1], "%me%", owl_zephyr_get_sender());
+	regexstr=owl_text_substitute(argv[i+1], "%me%", owl_zephyr_get_sender());
 	owl_filterelement_create_re(fe, argv[i], regexstr);
 	owl_free(regexstr);
 	i++;
+      } else if (!strcasecmp(argv[i], "filter")) {
+	owl_filterelement_create_filter(fe, argv[i+1]);
+	i++;
+      } else if (!strcasecmp(argv[i], "perl")) {
+	owl_filterelement_create_perl(fe, argv[i+1]);
+	i++;
       } else {
 	error=1;
       }
@@ -84,8 +98,29 @@ int owl_filter_init(owl_filter *f, char *name, int argc, char **argv)
       owl_filter_free(f);
       return(-1);
     }
-
   }
+  
+  /* Are we trying to use the filter we're creating?  Bad. */
+  owl_list_create(&list);
+  _owl_filter_get_subfilter_names(f, &list);
+  j=owl_list_get_size(&list);
+  for (i=0; i<j; i++) {
+    if (!strcasecmp(name, owl_list_get_element(&list, i))) {
+      owl_function_error("Filter loop.");
+      owl_filter_free(f);
+      owl_list_free_all(&list, owl_free);
+      return(-1);
+    }
+  }
+  owl_list_free_all(&list, owl_free);
+
+  /* Now check for more subtle recursion. */
+  if (owl_filter_is_toodeep(f)) {
+    owl_function_error("Filter loop or exceeds recursion depth");
+    owl_filter_free(f);
+    return(-1);
+  }
+  
   return(0);
 }
 
@@ -160,7 +195,9 @@ int owl_filter_message_match(owl_filter *f, owl_message *m)
     } else if (!strcasecmp(field, "realm")) {
       match=owl_message_get_realm(m);
     } else if (!strcasecmp(field, "type")) {
-      match=owl_message_type_to_string(m);
+      match=owl_message_get_type(m);
+    } else if (!strcasecmp(field, "hostname")) {
+      match=owl_message_get_hostname(m);
     } else if (!strcasecmp(field, "direction")) {
       if (owl_message_is_direction_out(m)) {
 	match="out";
@@ -180,7 +217,7 @@ int owl_filter_message_match(owl_filter *f, owl_message *m)
 	match="none";
       }
     }
-    
+
     tmp=owl_regex_compare(owl_filterelement_get_re(fe), match);
     if (!tmp) {
       owl_list_replace_element(&work_fes, i, owl_global_get_filterelement_true(&g));
@@ -189,6 +226,50 @@ int owl_filter_message_match(owl_filter *f, owl_message *m)
     }
   }
 
+  /* now subfilters and perl functions */
+  for (i=0; i<j; i++) {
+    owl_filter *subfilter;
+			   
+    fe=owl_list_get_element(&work_fes, i);
+
+    if (owl_filterelement_is_filter(fe)) {
+
+      subfilter=owl_global_get_filter(&g, owl_filterelement_get_filtername(fe));
+      if (!subfilter) {
+	/* the filter does not exist, maybe because it was deleted.
+	 * Default to not matching
+	 */
+	owl_list_replace_element(&work_fes, i, owl_global_get_filterelement_false(&g));
+      } else if (owl_filter_message_match(subfilter, m)) {
+	owl_list_replace_element(&work_fes, i, owl_global_get_filterelement_true(&g));
+      } else {
+	owl_list_replace_element(&work_fes, i, owl_global_get_filterelement_false(&g));
+      }
+
+    } else if (owl_filterelement_is_perl(fe)) {
+      char *subname, *perlrv;
+      int   tf=0;
+
+      subname = owl_filterelement_get_filtername(fe);
+      if (!owl_perlconfig_is_function(subname)) {
+	owl_list_replace_element(&work_fes, i, owl_global_get_filterelement_false(&g));
+	continue;
+      }
+      perlrv = owl_perlconfig_call_with_message(subname, m);
+      if (perlrv) {
+	if (0 == strcmp(perlrv, "1")) {
+	  tf=1;
+	}
+	owl_free(perlrv);
+      } 
+      if (tf) {
+	owl_list_replace_element(&work_fes, i, owl_global_get_filterelement_true(&g));
+      } else {
+	owl_list_replace_element(&work_fes, i, owl_global_get_filterelement_false(&g));
+      }
+    }
+  }
+
   /* call the recrsive helper */
   i=_owl_filter_message_match_recurse(f, m, &work_fes, 0, owl_list_get_size(&(f->fes))-1);
 
@@ -392,6 +473,7 @@ void owl_filter_print(owl_filter *f, char *out)
   strcat(out, "\n");
 }
 
+/* Return 1 if the filters 'a' and 'b' are equivalent, 0 otherwise */
 int owl_filter_equiv(owl_filter *a, owl_filter *b)
 {
   char buff[LINE], buff2[LINE];
@@ -403,6 +485,102 @@ int owl_filter_equiv(owl_filter *a, owl_filter *b)
   return(0);
 }
 
+/* Private
+ * 'list' should already be allocated and initialized
+ * This function places into list the string names of all filters
+ * used in the filter expression for 'f'.
+ * Caller must do a full free on 'list', including elements.
+ */
+void _owl_filter_get_subfilter_names(owl_filter *f, owl_list *list)
+{
+  int i, j;
+  owl_filterelement *fe;
+
+  j=owl_list_get_size(&(f->fes));
+  for (i=0; i<j; i++) {
+    fe=owl_list_get_element(&(f->fes), i);
+    if (owl_filterelement_is_filter(fe)) {
+      owl_list_append_element(list, owl_strdup(owl_filterelement_get_filtername(fe)));
+    }
+  }
+}
+
+int owl_filter_is_toodeep(owl_filter *f)
+{
+  owl_list seen, tocheck, tmp;
+  int i, j, x, y;
+  owl_filter *subfilter;
+
+  owl_list_create(&seen);
+  owl_list_create(&tocheck);
+  owl_list_create(&tmp);
+
+  /* seed 'tocheck' with the first set of filters */
+  _owl_filter_get_subfilter_names(f, &tmp);
+  j=owl_list_get_size(&tmp);
+  for (i=0; i<j; i++) {
+    owl_util_list_add_unique_string(&tocheck, owl_list_get_element(&tmp, i));
+  }
+  owl_list_free_all(&tmp, owl_free);
+  owl_list_create(&tmp);
+
+  /* add this list to the 'seen' list */
+  owl_list_append_element(&seen, owl_strdup(owl_filter_get_name(f)));
+
+  for (i=0; i<OWL_FILTER_MAXRECURSE; i++) {
+    /* if anything in 'tocheck' is in 'seen' we have a loop */
+    if (owl_util_common_strings_in_lists(&tocheck, &seen)) {
+      owl_list_free_all(&tmp, owl_free);
+      owl_list_free_all(&tocheck, owl_free);
+      owl_list_free_all(&seen, owl_free);
+      return(1);
+    }
+
+    /* if there's nothing to check, we're done */
+    y=owl_list_get_size(&tocheck);
+    if (y==0) {
+      owl_list_free_all(&tmp, owl_free);
+      owl_list_free_all(&tocheck, owl_free);
+      owl_list_free_all(&seen, owl_free);
+      return(0);
+    }
+
+    /* add everything in 'tocheck' to the 'seen' list */
+    /* y=owl_list_get_size(&tocheck); */
+    for (x=0; x<y; x++) {
+      owl_list_append_element(&seen, owl_strdup(owl_list_get_element(&tocheck, x)));
+    }
+
+    /* make a new list into 'tmp' with the children of 'tocheck' */
+    /* y=owl_list_get_size(&tocheck); */
+    for (x=0; x<y; x++) {
+      subfilter=owl_global_get_filter(&g, owl_list_get_element(&tocheck, x));
+      if (!subfilter) return(0);
+      _owl_filter_get_subfilter_names(subfilter, &tmp);
+    }
+
+    /* clean out 'tocheck' */
+    owl_list_free_all(&tocheck, owl_free);
+    owl_list_create(&tocheck);
+
+    /* put 'tmp' into 'tocheck' */
+    y=owl_list_get_size(&tmp);
+    for (x=0; x<y; x++) {
+      owl_util_list_add_unique_string(&tocheck, owl_list_get_element(&tmp, x));
+    }
+
+    /* clean out 'tmp' */
+    owl_list_free_all(&tmp, owl_free);
+    owl_list_create(&tmp);
+  }
+
+  owl_list_free_all(&tmp, owl_free);
+  owl_list_free_all(&tocheck, owl_free);
+  owl_list_free_all(&seen, owl_free);
+
+  return(1);
+}
+
 void owl_filter_free(owl_filter *f)
 {
   void (*func)();
