@@ -1,6 +1,6 @@
 #include "owl.h"
 
-static const char fileIdent[] = "$Id: filterelement.c,v 1.3 2003/06/22 17:40:49 kretch Exp $";
+static const char fileIdent[] = "$Id: filterelement.c,v 1.5 2003/12/26 19:50:10 nygren Exp $";
 
 #define OWL_FILTERELEMENT_NULL        0
 #define OWL_FILTERELEMENT_TRUE        1
@@ -11,66 +11,84 @@ static const char fileIdent[] = "$Id: filterelement.c,v 1.3 2003/06/22 17:40:49
 #define OWL_FILTERELEMENT_OR          6
 #define OWL_FILTERELEMENT_NOT         7
 #define OWL_FILTERELEMENT_RE          8
-
+#define OWL_FILTERELEMENT_FILTER      9
+#define OWL_FILTERELEMENT_PERL       10
 
 void owl_filterelement_create_null(owl_filterelement *fe)
 {
   fe->type=OWL_FILTERELEMENT_NULL;
   fe->field=NULL;
+  fe->filtername=NULL;
 }
 
 void owl_filterelement_create_openbrace(owl_filterelement *fe)
 {
+  owl_filterelement_create_null(fe);
   fe->type=OWL_FILTERELEMENT_OPENBRACE;
-  fe->field=NULL;
 }
 
 void owl_filterelement_create_closebrace(owl_filterelement *fe)
 {
+  owl_filterelement_create_null(fe);
   fe->type=OWL_FILTERELEMENT_CLOSEBRACE;
-  fe->field=NULL;
 }
 
 void owl_filterelement_create_and(owl_filterelement *fe)
 {
+  owl_filterelement_create_null(fe);
   fe->type=OWL_FILTERELEMENT_AND;
-  fe->field=NULL;
 }
 
 void owl_filterelement_create_or(owl_filterelement *fe)
 {
+  owl_filterelement_create_null(fe);
   fe->type=OWL_FILTERELEMENT_OR;
-  fe->field=NULL;
 }
 
 void owl_filterelement_create_not(owl_filterelement *fe)
 {
+  owl_filterelement_create_null(fe);
   fe->type=OWL_FILTERELEMENT_NOT;
-  fe->field=NULL;
 }
 
 void owl_filterelement_create_true(owl_filterelement *fe)
 {
+  owl_filterelement_create_null(fe);
   fe->type=OWL_FILTERELEMENT_TRUE;
-  fe->field=NULL;
 }
 
 void owl_filterelement_create_false(owl_filterelement *fe)
 {
+  owl_filterelement_create_null(fe);
   fe->type=OWL_FILTERELEMENT_FALSE;
-  fe->field=NULL;
 }
 
 void owl_filterelement_create_re(owl_filterelement *fe, char *field, char *re)
 {
+  owl_filterelement_create_null(fe);
   fe->type=OWL_FILTERELEMENT_RE;
   fe->field=owl_strdup(field);
   owl_regex_create(&(fe->re), re);
 }
 
+void owl_filterelement_create_filter(owl_filterelement *fe, char *name)
+{
+  owl_filterelement_create_null(fe);
+  fe->type=OWL_FILTERELEMENT_FILTER;
+  fe->filtername=owl_strdup(name);
+}
+
+void owl_filterelement_create_perl(owl_filterelement *fe, char *name)
+{
+  owl_filterelement_create_null(fe);
+  fe->type=OWL_FILTERELEMENT_PERL;
+  fe->filtername=owl_strdup(name);
+}
+
 void owl_filterelement_free(owl_filterelement *fe)
 {
   if (fe->field) owl_free(fe->field);
+  if (fe->filtername) owl_free(fe->filtername);
 }
 
 int owl_filterelement_is_null(owl_filterelement *fe)
@@ -127,31 +145,50 @@ int owl_filterelement_is_re(owl_filterelement *fe)
   return(0);
 }
 
+int owl_filterelement_is_perl(owl_filterelement *fe)
+{
+  if (fe->type==OWL_FILTERELEMENT_PERL) return(1);
+  return(0);
+}
+
 owl_regex *owl_filterelement_get_re(owl_filterelement *fe)
 {
   return(&(fe->re));
 }
 
+int owl_filterelement_is_filter(owl_filterelement *fe)
+{
+  if (fe->type==OWL_FILTERELEMENT_FILTER) return(1);
+  return(0);
+}
+
 char *owl_filterelement_get_field(owl_filterelement *fe)
 {
-  return(fe->field);
+  if (fe->field) return(fe->field);
+  return("unknown-field");
+}
+
+char *owl_filterelement_get_filtername(owl_filterelement *fe)
+{
+  if (fe->filtername) return(fe->filtername);
+  return("unknown-filter");
 }
 
 int owl_filterelement_is_value(owl_filterelement *fe)
 {
   if ( (fe->type==OWL_FILTERELEMENT_TRUE) ||
        (fe->type==OWL_FILTERELEMENT_FALSE) ||
-       (fe->type==OWL_FILTERELEMENT_RE) ) {
+       (fe->type==OWL_FILTERELEMENT_RE) ||
+       (fe->type==OWL_FILTERELEMENT_PERL) ||
+       (fe->type==OWL_FILTERELEMENT_FILTER)) {
     return(1);
   }
   return(0);
 }
 
-
+/* caller must free the return */
 char *owl_filterelement_to_string(owl_filterelement *fe)
 {
-  /* return must be freed by caller */
-  
   if (owl_filterelement_is_openbrace(fe)) {
     return(owl_strdup("( "));
   } else if (owl_filterelement_is_closebrace(fe)) {
@@ -167,10 +204,12 @@ char *owl_filterelement_to_string(owl_filterelement *fe)
   } else if (owl_filterelement_is_false(fe)) {
     return(owl_strdup("false "));
   } else if (owl_filterelement_is_re(fe)) {
-    char *buff;
-    buff=owl_malloc(LINE);
-    sprintf(buff, "%s %s ", fe->field, owl_regex_get_string(&(fe->re)));
-    return(buff);
+    return(owl_sprintf("%s %s ", fe->field, owl_regex_get_string(&(fe->re))));
+  } else if (owl_filterelement_is_filter(fe)) {
+    return(owl_sprintf("filter %s ", fe->filtername));
+  } else if (owl_filterelement_is_perl(fe)) {
+    return(owl_sprintf("perl %s ", fe->filtername));
   }
-  return(owl_strdup(""));
+
+  return(owl_strdup("?"));
 }
