@@ -8,7 +8,7 @@
 #include <time.h>
 #include "owl.h"
 
-static const char fileIdent[] = "$Id: global.c,v 1.24 2003/10/03 23:34:02 kretch Exp $";
+static const char fileIdent[] = "$Id: global.c,v 1.32 2004/03/01 17:43:22 nygren Exp $";
 
 #ifndef MAXHOSTNAMELEN
 #define MAXHOSTNAMELEN 256
@@ -49,6 +49,7 @@ void owl_global_init(owl_global *g) {
   owl_keyhandler_init(&g->kh);
   owl_keys_setup_keymaps(&g->kh);
 
+  owl_list_create(&(g->muxevents));
   owl_list_create(&(g->filterlist));
   owl_list_create(&(g->puntlist));
   owl_list_create(&(g->messagequeue));
@@ -74,6 +75,7 @@ void owl_global_init(owl_global *g) {
   owl_global_set_no_have_config(g);
   owl_history_init(&(g->msghist));
   owl_history_init(&(g->cmdhist));
+  owl_history_set_norepeats(&(g->cmdhist));
   g->nextmsgid=0;
 
   owl_filterelement_create_true(&(g->fe_true));
@@ -101,6 +103,12 @@ void owl_global_init(owl_global *g) {
   g->havezephyr=0;
   g->haveaim=0;
   owl_global_set_no_doaimevents(g);
+
+  owl_errqueue_init(&(g->errqueue));
+  g->got_err_signal=0;
+
+  owl_zbuddylist_create(&(g->zbuddies));
+  owl_timer_create_countdown(&(g->zephyr_buddycheck_timer), 60*3);
 }
 
 void _owl_global_setup_windows(owl_global *g) {
@@ -515,6 +523,12 @@ owl_history *owl_global_get_cmd_history(owl_global *g) {
   return(&(g->cmdhist));
 }
 
+/* muxevents */
+
+owl_muxevents *owl_global_get_muxevents(owl_global *g) {
+  return(&(g->muxevents));
+}
+
 /* filterlist */
 
 owl_list *owl_global_get_filterlist(owl_global *g) {
@@ -712,15 +726,19 @@ void owl_global_set_no_doaimevents(owl_global *g)
   g->aim_doprocessing=0;
 }
 
-
 aim_session_t *owl_global_get_aimsess(owl_global *g)
 {
   return(&(g->aimsess));
 }
 
-aim_conn_t *owl_global_get_waitingconn(owl_global *g)
+aim_conn_t *owl_global_get_bosconn(owl_global *g)
 {
-  return(&(g->waitingconn));
+  return(&(g->bosconn));
+}
+
+void owl_global_set_bossconn(owl_global *g, aim_conn_t *conn)
+{
+  g->bosconn=*conn;
 }
 
 int owl_global_is_aimnop_time(owl_global *g)
@@ -831,3 +849,39 @@ owl_timer *owl_global_get_aim_buddyinfo_timer(owl_global *g)
 {
   return(&(g->aim_buddyinfo_timer));
 }
+
+owl_errqueue *owl_global_get_errqueue(owl_global *g)
+{
+  return(&(g->errqueue));
+}
+
+void owl_global_set_errsignal(owl_global *g, int signum, siginfo_t *siginfo)
+{
+  g->got_err_signal = signum;
+  if (siginfo) {
+    g->err_signal_info = *siginfo;
+  } else {
+    memset(&(g->err_signal_info), 0, sizeof(siginfo_t));
+  }
+}
+
+int owl_global_get_errsignal_and_clear(owl_global *g, siginfo_t *siginfo)
+{
+  int signum;
+  if (siginfo && g->got_err_signal) {
+    *siginfo = g->err_signal_info;
+  } 
+  signum = g->got_err_signal;
+  g->got_err_signal = 0;
+  return signum;
+}
+
+owl_timer *owl_global_get_zephyr_buddycheck_timer(owl_global *g)
+{
+  return(&(g->zephyr_buddycheck_timer));
+}
+
+owl_zbuddylist *owl_global_get_zephyr_buddylist(owl_global *g)
+{
+  return(&(g->zbuddies));
+}
