@@ -4,9 +4,60 @@
 #include <ctype.h>
 #include <sys/param.h>
 
-static const char fileIdent[] = "$Id: logging.c,v 1.11 2003/07/03 00:15:05 kretch Exp $";
+static const char fileIdent[] = "$Id: logging.c,v 1.22 2004/02/23 06:15:42 kretch Exp $";
 
-void owl_log_outgoing_zephyr(char *to, char *text) {
+void owl_log_outgoing_zephyr(char *to, char *text)
+{
+  FILE *file;
+  char filename[MAXPATHLEN], *logpath;
+  char *tobuff, *ptr="";
+
+  if (owl_global_get_loggingdirection(&g)==OWL_LOGGING_DIRECTION_IN) return;
+
+  tobuff=owl_malloc(strlen(to)+20);
+  strcpy(tobuff, to);
+
+  /* chop off a local realm */
+  ptr=strchr(tobuff, '@');
+  if (ptr && !strncmp(ptr+1, owl_zephyr_get_realm(), strlen(owl_zephyr_get_realm()))) {
+    *ptr='\0';
+  }
+
+  /* expand ~ in path names */
+  logpath = owl_text_substitute(owl_global_get_logpath(&g), "~", 
+				owl_global_get_homedir(&g));
+
+  snprintf(filename, MAXPATHLEN, "%s/%s", logpath, tobuff);
+  file=fopen(filename, "a");
+  if (!file) {
+    owl_function_error("Unable to open file for outgoing logging");
+    owl_free(logpath);
+    return;
+  }
+  fprintf(file, "OUTGOING (owl): %s\n%s\n", tobuff, text);
+  if (text[strlen(text)-1]!='\n') {
+    fprintf(file, "\n");
+  }
+  fclose(file);
+
+  snprintf(filename, MAXPATHLEN, "%s/all", logpath);
+  owl_free(logpath);
+  file=fopen(filename, "a");
+  if (!file) {
+    owl_function_error("Unable to open file for outgoing logging");
+    return;
+  }
+  fprintf(file, "OUTGOING (owl): %s\n%s\n", tobuff, text);
+  if (text[strlen(text)-1]!='\n') {
+    fprintf(file, "\n");
+  }
+  fclose(file);
+
+  owl_free(tobuff);
+}
+
+void owl_log_outgoing_zephyr_error(char *to, char *text)
+{
   FILE *file;
   char filename[MAXPATHLEN], *logpath;
   char *tobuff, *ptr="";
@@ -21,13 +72,59 @@ void owl_log_outgoing_zephyr(char *to, char *text) {
   }
 
   /* expand ~ in path names */
-  logpath = owl_util_substitute(owl_global_get_logpath(&g), "~", 
+  logpath = owl_text_substitute(owl_global_get_logpath(&g), "~", 
 				owl_global_get_homedir(&g));
 
   snprintf(filename, MAXPATHLEN, "%s/%s", logpath, tobuff);
   file=fopen(filename, "a");
   if (!file) {
-    owl_function_makemsg("Unable to open file for outgoing logging");
+    owl_function_error("Unable to open file for outgoing logging");
+    owl_free(logpath);
+    return;
+  }
+  fprintf(file, "ERROR (owl): %s\n%s\n", tobuff, text);
+  if (text[strlen(text)-1]!='\n') {
+    fprintf(file, "\n");
+  }
+  fclose(file);
+
+  snprintf(filename, MAXPATHLEN, "%s/all", logpath);
+  owl_free(logpath);
+  file=fopen(filename, "a");
+  if (!file) {
+    owl_function_error("Unable to open file for outgoing logging");
+    return;
+  }
+  fprintf(file, "ERROR (owl): %s\n%s\n", tobuff, text);
+  if (text[strlen(text)-1]!='\n') {
+    fprintf(file, "\n");
+  }
+  fclose(file);
+
+  owl_free(tobuff);
+}
+
+void owl_log_outgoing_aim(char *to, char *text)
+{
+  FILE *file;
+  char filename[MAXPATHLEN], *logpath;
+  char *tobuff, *normalto;
+
+  if (owl_global_get_loggingdirection(&g)==OWL_LOGGING_DIRECTION_IN) return;
+
+  normalto=owl_aim_normalize_screenname(to);
+  downstr(normalto);
+  tobuff=owl_sprintf("aim:%s", normalto);
+  owl_free(normalto);
+
+  /* expand ~ in path names */
+  logpath = owl_text_substitute(owl_global_get_logpath(&g), "~", 
+				owl_global_get_homedir(&g));
+
+  snprintf(filename, MAXPATHLEN, "%s/%s", logpath, tobuff);
+  file=fopen(filename, "a");
+  if (!file) {
+    owl_function_error("Unable to open file for outgoing logging");
     owl_free(logpath);
     return;
   }
@@ -41,7 +138,7 @@ void owl_log_outgoing_zephyr(char *to, char *text) {
   owl_free(logpath);
   file=fopen(filename, "a");
   if (!file) {
-    owl_function_makemsg("Unable to open file for outgoing logging");
+    owl_function_error("Unable to open file for outgoing logging");
     return;
   }
   fprintf(file, "OUTGOING (owl): %s\n%s\n", tobuff, text);
@@ -53,21 +150,24 @@ void owl_log_outgoing_zephyr(char *to, char *text) {
   owl_free(tobuff);
 }
 
-void owl_log_outgoing_aim(char *to, char *text) {
+void owl_log_outgoing_loopback(char *text)
+{
   FILE *file;
   char filename[MAXPATHLEN], *logpath;
   char *tobuff;
 
-  tobuff=owl_sprintf("aim:%s", to);
+  if (owl_global_get_loggingdirection(&g)==OWL_LOGGING_DIRECTION_IN) return;
+
+  tobuff=owl_sprintf("loopback");
 
   /* expand ~ in path names */
-  logpath = owl_util_substitute(owl_global_get_logpath(&g), "~", 
+  logpath = owl_text_substitute(owl_global_get_logpath(&g), "~", 
 				owl_global_get_homedir(&g));
 
   snprintf(filename, MAXPATHLEN, "%s/%s", logpath, tobuff);
   file=fopen(filename, "a");
   if (!file) {
-    owl_function_makemsg("Unable to open file for outgoing logging");
+    owl_function_error("Unable to open file for outgoing logging");
     owl_free(logpath);
     return;
   }
@@ -81,7 +181,7 @@ void owl_log_outgoing_aim(char *to, char *text) {
   owl_free(logpath);
   file=fopen(filename, "a");
   if (!file) {
-    owl_function_makemsg("Unable to open file for outgoing logging");
+    owl_function_error("Unable to open file for outgoing logging");
     return;
   }
   fprintf(file, "OUTGOING (owl): %s\n%s\n", tobuff, text);
@@ -93,11 +193,17 @@ void owl_log_outgoing_aim(char *to, char *text) {
   owl_free(tobuff);
 }
 
-void owl_log_incoming(owl_message *m) {
+void owl_log_incoming(owl_message *m)
+{
   FILE *file, *allfile;
   char filename[MAXPATHLEN], allfilename[MAXPATHLEN], *logpath;
   char *frombuff=NULL, *from=NULL, *buff=NULL, *ptr;
   int len, ch, i, personal;
+
+  if (owl_global_get_loggingdirection(&g)==OWL_LOGGING_DIRECTION_OUT) return;
+
+  /* skip login/logout messages if appropriate */
+  if (!owl_global_is_loglogins(&g) && owl_message_is_loginout(m)) return;
       
   /* check for nolog */
   if (!strcasecmp(owl_message_get_opcode(m), "nolog") ||
@@ -138,7 +244,13 @@ void owl_log_incoming(owl_message *m) {
     }
   } else if (owl_message_is_type_aim(m)) {
     /* we do not yet handle chat rooms */
-    from=frombuff=owl_sprintf("aim:%s", owl_message_get_sender(m));
+    char *normalto;
+    normalto=owl_aim_normalize_screenname(owl_message_get_sender(m));
+    downstr(normalto);
+    from=frombuff=owl_sprintf("aim:%s", normalto);
+    owl_free(normalto);
+  } else if (owl_message_is_type_loopback(m)) {
+    from=frombuff=owl_strdup("loopback");
   } else {
     from=frombuff=owl_strdup("unknown");
   }
@@ -163,13 +275,13 @@ void owl_log_incoming(owl_message *m) {
 
   /* create the filename (expanding ~ in path names) */
   if (personal) {
-    logpath = owl_util_substitute(owl_global_get_logpath(&g), "~", 
+    logpath = owl_text_substitute(owl_global_get_logpath(&g), "~", 
 				  owl_global_get_homedir(&g));
     snprintf(filename, MAXPATHLEN, "%s/%s", logpath, from);
     snprintf(allfilename, MAXPATHLEN, "%s/all", logpath);
 
   } else {
-    logpath = owl_util_substitute(owl_global_get_classlogpath(&g), "~", 
+    logpath = owl_text_substitute(owl_global_get_classlogpath(&g), "~", 
 				owl_global_get_homedir(&g));
 
     snprintf(filename, MAXPATHLEN, "%s/%s", logpath, from);
@@ -178,7 +290,7 @@ void owl_log_incoming(owl_message *m) {
   
   file=fopen(filename, "a");
   if (!file) {
-    owl_function_makemsg("Unable to open file for incoming logging");
+    owl_function_error("Unable to open file for incoming logging");
     return;
   }
 
@@ -186,7 +298,7 @@ void owl_log_incoming(owl_message *m) {
   if (personal) {
     allfile=fopen(allfilename, "a");
     if (!allfile) {
-      owl_function_makemsg("Unable to open file for incoming logging");
+      owl_function_error("Unable to open file for incoming logging");
       fclose(file);
       return;
     }
@@ -206,7 +318,7 @@ void owl_log_incoming(owl_message *m) {
     memcpy(buff, ptr, i);
     buff[i]='\0';
     fprintf(file, "From: %s <%s>\n\n", buff, tmp);
-    fprintf(file, "%s\n", owl_message_get_body(m));
+    fprintf(file, "%s\n\n", owl_message_get_body(m));
     owl_free(tmp);
   } else if (owl_message_is_type_aim(m) && !owl_message_is_loginout(m)) {
     fprintf(file, "From: <%s> To: <%s>\n", owl_message_get_sender(m), owl_message_get_recipient(m));
@@ -217,7 +329,12 @@ void owl_log_incoming(owl_message *m) {
     fprintf(file, "Time: %s\n\n", owl_message_get_timestr(m));
     if (owl_message_is_login(m)) fprintf(file, "LOGIN\n\n");
     if (owl_message_is_logout(m)) fprintf(file, "LOGOUT\n\n");
+  } else {
+    fprintf(file, "From: <%s> To: <%s>\n", owl_message_get_sender(m), owl_message_get_recipient(m));
+    fprintf(file, "Time: %s\n\n", owl_message_get_timestr(m));
+    fprintf(file, "%s\n\n", owl_message_get_body(m));
   }
+
   fclose(file);
 
   /* if it's a personal message, also write to the 'all' file */
@@ -231,7 +348,7 @@ void owl_log_incoming(owl_message *m) {
       fprintf(allfile, "\n");
       fprintf(allfile, "Time: %s Host: %s\n", owl_message_get_timestr(m), owl_message_get_hostname(m));
       fprintf(allfile, "From: %s <%s>\n\n", buff, tmp);
-      fprintf(allfile, "%s\n", owl_message_get_body(m));
+      fprintf(allfile, "%s\n\n", owl_message_get_body(m));
       owl_free(tmp);
     } else if (owl_message_is_type_aim(m) && !owl_message_is_loginout(m)) {
       fprintf(allfile, "From: <%s> To: <%s>\n", owl_message_get_sender(m), owl_message_get_recipient(m));
@@ -242,6 +359,10 @@ void owl_log_incoming(owl_message *m) {
       fprintf(allfile, "Time: %s\n\n", owl_message_get_timestr(m));
       if (owl_message_is_login(m)) fprintf(allfile, "LOGIN\n\n");
       if (owl_message_is_logout(m)) fprintf(allfile, "LOGOUT\n\n");
+    } else {
+      fprintf(allfile, "From: <%s> To: <%s>\n", owl_message_get_sender(m), owl_message_get_recipient(m));
+      fprintf(allfile, "Time: %s\n\n", owl_message_get_timestr(m));
+      fprintf(allfile, "%s\n\n", owl_message_get_body(m));
     }
     fclose(allfile);
   }
