@@ -10,12 +10,10 @@
 #include <time.h>
 #include "owl.h"
 
-static const char fileIdent[] = "$Id: message.c,v 1.41 2003/10/10 21:12:20 kretch Exp $";
+static const char fileIdent[] = "$Id: message.c,v 1.49 2003/12/23 22:07:43 kretch Exp $";
 
 void owl_message_init(owl_message *m)
 {
-  time_t t;
-
   m->id=owl_global_get_nextmsgid(&g);
   m->type=OWL_MESSAGE_TYPE_GENERIC;
   owl_message_set_direction_none(m);
@@ -27,17 +25,19 @@ void owl_message_init(owl_message *m)
   owl_list_create(&(m->attributes));
   
   /* save the time */
-  t=time(NULL);
-  m->time=owl_strdup(ctime(&t));
-  m->time[strlen(m->time)-1]='\0';
+  m->time=time(NULL);
+  m->timestr=owl_strdup(ctime(&(m->time)));
+  m->timestr[strlen(m->timestr)-1]='\0';
+
+  /* initialize the fmtext */
   owl_fmtext_init_null(&(m->fmtext));
 }
 
+/* add the named attribute to the message.  If an attribute with the
+ * name already exists, replace the old value with the new value
+ */
 void owl_message_set_attribute(owl_message *m, char *attrname, char *attrvalue)
 {
-  /* add the named attribute to the message.  If an attribute with the
-     name already exists, replace the old value with the new value */
-
   int i, j;
   owl_pair *p;
 
@@ -60,10 +60,11 @@ void owl_message_set_attribute(owl_message *m, char *attrname, char *attrvalue)
   owl_list_append_element(&(m->attributes), p);
 }
 
+/* return the value associated with the named attribute, or NULL if
+ * the attribute does not exist
+ */
 char *owl_message_get_attribute_value(owl_message *m, char *attrname)
 {
-  /* return the value associated with the named attribute, or NULL if
-     the attribute does not exist */
   int i, j;
   owl_pair *p;
 
@@ -304,7 +305,20 @@ int owl_message_is_private(owl_message *m)
 
 char *owl_message_get_timestr(owl_message *m)
 {
-  return(m->time);
+  if (m->timestr) return(m->timestr);
+  return("");
+}
+
+/* caller must free the return */
+char *owl_message_get_shorttimestr(owl_message *m)
+{
+  struct tm *tmstruct;
+  char *out;
+
+  tmstruct=localtime(&(m->time));
+  out=owl_sprintf("%2.2i:%2.2i", tmstruct->tm_hour, tmstruct->tm_min);
+  if (out) return(out);
+  return("??:??");
 }
 
 void owl_message_set_type_admin(owl_message *m)
@@ -312,6 +326,11 @@ void owl_message_set_type_admin(owl_message *m)
   m->type=OWL_MESSAGE_TYPE_ADMIN;
 }
 
+void owl_message_set_type_loopback(owl_message *m)
+{
+  m->type=OWL_MESSAGE_TYPE_LOOPBACK;
+}
+
 void owl_message_set_type_zephyr(owl_message *m)
 {
   m->type=OWL_MESSAGE_TYPE_ZEPHYR;
@@ -328,6 +347,12 @@ int owl_message_is_type_admin(owl_message *m)
   return(0);
 }
 
+int owl_message_is_type_loopback(owl_message *m)
+{
+  if (m->type==OWL_MESSAGE_TYPE_LOOPBACK) return(1);
+  return(0);
+}
+
 int owl_message_is_type_zephyr(owl_message *m)
 {
   if (m->type==OWL_MESSAGE_TYPE_ZEPHYR) return(1);
@@ -340,24 +365,18 @@ int owl_message_is_type_aim(owl_message *m)
   return(0);
 }
 
+int owl_message_is_pseudo(owl_message *m)
+{
+  if (owl_message_get_attribute_value(m, "pseudo")) return(1);
+  return(0);
+}
+
 int owl_message_is_type_generic(owl_message *m)
 {
   if (m->type==OWL_MESSAGE_TYPE_GENERIC) return(1);
   return(0);
 }
 
-char *owl_message_type_to_string(owl_message *m)
-{
-  if (m->type==OWL_MESSAGE_TYPE_ADMIN) return("admin");
-  if (m->type==OWL_MESSAGE_TYPE_GENERIC) return("generic");
-  if (m->type==OWL_MESSAGE_TYPE_ZEPHYR) return("zephyr");
-  if (m->type==OWL_MESSAGE_TYPE_AIM) return("aim");
-  if (m->type==OWL_MESSAGE_TYPE_JABBER) return("jabber");
-  if (m->type==OWL_MESSAGE_TYPE_ICQ) return("icq");
-  if (m->type==OWL_MESSAGE_TYPE_MSN) return("msn");
-  return("unknown");
-}
-
 char *owl_message_get_text(owl_message *m)
 {
   return(owl_fmtext_get_text(&(m->fmtext)));
@@ -449,7 +468,6 @@ char *owl_message_get_hostname(owl_message *m)
   return(m->hostname);
 }
 
-
 void owl_message_curs_waddstr(owl_message *m, WINDOW *win, int aline, int bline, int acol, int bcol, int color)
 {
   owl_fmtext a, b;
@@ -596,6 +614,8 @@ char *owl_message_get_type(owl_message *m) {
     return("yahoo");
   case OWL_MESSAGE_TYPE_MSN:
     return("msn");
+  case OWL_MESSAGE_TYPE_LOOPBACK:
+    return("loopback");
   default:
     return("unknown");
   }
@@ -678,11 +698,21 @@ void owl_message_create_admin(owl_message *m, char *header, char *text)
   owl_message_set_attribute(m, "adminheader", header); /* just a hack for now */
 }
 
+/* caller should set the direction */
+void owl_message_create_loopback(owl_message *m, char *text)
+{
+  owl_message_init(m);
+  owl_message_set_type_loopback(m);
+  owl_message_set_body(m, text);
+  owl_message_set_sender(m, "loopsender");
+  owl_message_set_recipient(m, "looprecip");
+  owl_message_set_isprivate(m);
+}
+
 #ifdef HAVE_LIBZEPHYR
 void owl_message_create_from_znotice(owl_message *m, ZNotice_t *n)
 {
   struct hostent *hent;
-  int k;
   char *ptr, *tmp, *tmp2;
 
   owl_message_init(m);
@@ -693,10 +723,14 @@ void owl_message_create_from_znotice(owl_message *m, ZNotice_t *n)
   /* first save the full notice */
   memcpy(&(m->notice), n, sizeof(ZNotice_t));
 
-  /* a little gross, we'll reaplace \r's with ' ' for now */
+  /* a little gross, we'll replace \r's with ' ' for now */
   owl_zephyr_hackaway_cr(&(m->notice));
   
-  m->delete=0;
+  /* save the time, we need to nuke the string saved by message_init */
+  if (m->timestr) owl_free(m->timestr);
+  m->time=n->z_time.tv_sec;
+  m->timestr=owl_strdup(ctime(&(m->time)));
+  m->timestr[strlen(m->timestr)-1]='\0';
 
   /* set other info */
   owl_message_set_sender(m, n->z_sender);
@@ -718,6 +752,16 @@ void owl_message_create_from_znotice(owl_message *m, ZNotice_t *n)
 
   /* Set the "isloginout" attribute if it's a login message */
   if (!strcasecmp(n->z_class, "login") || !strcasecmp(n->z_class, OWL_WEBZEPHYR_CLASS)) {
+    if (!strcasecmp(n->z_opcode, "user_login") || !strcasecmp(n->z_opcode, "user_logout")) {
+      tmp=owl_zephyr_get_field(n, 1);
+      owl_message_set_attribute(m, "loginhost", tmp);
+      owl_free(tmp);
+
+      tmp=owl_zephyr_get_field(n, 3);
+      owl_message_set_attribute(m, "logintty", tmp);
+      owl_free(tmp);
+    }
+
     if (!strcasecmp(n->z_opcode, "user_login")) {
       owl_message_set_islogin(m);
     } else if (!strcasecmp(n->z_opcode, "user_logout")) {
@@ -725,18 +769,22 @@ void owl_message_create_from_znotice(owl_message *m, ZNotice_t *n)
     }
   }
 
-  /* is the "isprivate" attribute if it's a private zephyr */
+  
+  /* set the "isprivate" attribute if it's a private zephyr */
   if (!strcasecmp(n->z_recipient, owl_zephyr_get_sender())) {
     owl_message_set_isprivate(m);
   }
 
+  /* set the "isauto" attribute if it's an autoreply */
+  if (!strcasecmp(n->z_message, "Automated reply:") ||
+      !strcasecmp(n->z_opcode, "auto")) {
+    owl_message_set_attribute(m, "isauto", "");
+  }
+
   m->zwriteline=strdup("");
 
   /* set the body */
-  ptr=owl_zephyr_get_message(n, &k);
-  tmp=owl_malloc(k+10);
-  memcpy(tmp, ptr, k);
-  tmp[k]='\0';
+  tmp=owl_zephyr_get_message(n);
   if (owl_global_is_newlinestrip(&g)) {
     tmp2=owl_util_stripnewlines(tmp);
     owl_message_set_body(m, tmp2);
@@ -770,10 +818,6 @@ void owl_message_create_from_znotice(owl_message *m, ZNotice_t *n)
   } else {
     strcpy(m->hostname, inet_ntoa(n->z_sender_addr));
   }
-
-  /* save the time */
-  m->time=owl_strdup(ctime((time_t *) &n->z_time.tv_sec));
-  m->time[strlen(m->time)-1]='\0';
 }
 #else
 void owl_message_create_from_znotice(owl_message *m, void *n)
@@ -781,6 +825,54 @@ void owl_message_create_from_znotice(owl_message *m, void *n)
 }
 #endif
 
+/* If 'direction' is '0' it is a login message, '1' is a logout message. */
+void owl_message_create_pseudo_zlogin(owl_message *m, int direction, char *user, char *host, char *time, char *tty)
+{
+  char *longuser, *ptr;
+
+#ifdef HAVE_LIBZEPHYR
+  memset(&(m->notice), 0, sizeof(ZNotice_t));
+#endif
+  
+  longuser=long_zuser(user);
+  
+  owl_message_init(m);
+  
+  owl_message_set_type_zephyr(m);
+  owl_message_set_direction_in(m);
+
+  owl_message_set_attribute(m, "pseudo", "");
+  owl_message_set_attribute(m, "loginhost", host ? host : "");
+  owl_message_set_attribute(m, "logintty", tty ? tty : "");
+
+  owl_message_set_sender(m, longuser);
+  owl_message_set_class(m, "LOGIN");
+  owl_message_set_instance(m, longuser);
+  owl_message_set_recipient(m, "");
+  if (direction==0) {
+    owl_message_set_opcode(m, "USER_LOGIN");
+    owl_message_set_islogin(m);
+  } else if (direction==1) {
+    owl_message_set_opcode(m, "USER_LOGOUT");
+    owl_message_set_islogout(m);
+  }
+
+  if ((ptr=strchr(longuser, '@'))!=NULL) {
+    owl_message_set_realm(m, ptr+1);
+  } else {
+    owl_message_set_realm(m, owl_zephyr_get_realm());
+  }
+
+  m->zwriteline=strdup("");
+
+  owl_message_set_body(m, "<uninitialized>");
+
+  /* save the hostname */
+  owl_function_debugmsg("create_pseudo_login: host is %s", host ? host : "");
+  strcpy(m->hostname, host ? host : "");
+  owl_free(longuser);
+}
+
 void owl_message_create_from_zwriteline(owl_message *m, char *line, char *body, char *zsig)
 {
   owl_zwrite z;
@@ -816,7 +908,6 @@ void owl_message_create_from_zwriteline(owl_message *m, char *line, char *body,
   owl_zwrite_free(&z);
 }
 
-
 void owl_message_pretty_zsig(owl_message *m, char *buff)
 {
   /* stick a one line version of the zsig in buff */
@@ -836,7 +927,7 @@ void owl_message_free(owl_message *m)
     ZFreeNotice(&(m->notice));
   }
 #endif
-  if (m->time) owl_free(m->time);
+  if (m->timestr) owl_free(m->timestr);
   if (m->zwriteline) owl_free(m->zwriteline);
 
   /* free all the attributes */
