@@ -1,7 +1,7 @@
 #include <string.h>
 #include "owl.h"
 
-static const char fileIdent[] = "$Id: regex.c,v 1.4 2003/06/22 17:40:49 kretch Exp $";
+static const char fileIdent[] = "$Id: regex.c,v 1.5 2003/10/26 21:11:37 kretch Exp $";
 
 void owl_regex_init(owl_regex *re)
 {
@@ -38,6 +38,16 @@ int owl_regex_create(owl_regex *re, char *string)
   return(0);
 }
 
+int owl_regex_create_quoted(owl_regex *re, char *string)
+{
+  char *quoted;
+  
+  quoted=owl_text_quote(string, OWL_REGEX_QUOTECHARS, OWL_REGEX_QUOTEWITH);
+  owl_regex_create(re, quoted);
+  owl_free(quoted);
+  return(0);
+}
+
 int owl_regex_compare(owl_regex *re, char *string)
 {
   int out, ret;
