@@ -2,10 +2,10 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <unistd.h>
 #include <ctype.h>
+#include <pwd.h>
 
-static const char fileIdent[] = "$Id: util.c,v 1.33 2003/07/06 22:42:34 nygren Exp $";
+static const char fileIdent[] = "$Id: util.c,v 1.46 2004/04/18 15:55:42 kretch Exp $";
 
 void sepbar(char *in)
 {
@@ -22,7 +22,11 @@ void sepbar(char *in)
 
   werase(sepwin);
   wattron(sepwin, A_REVERSE);
-  whline(sepwin, ACS_HLINE, owl_global_get_cols(&g));
+  if (owl_global_is_fancylines(&g)) {
+    whline(sepwin, ACS_HLINE, owl_global_get_cols(&g));
+  } else {
+    whline(sepwin, '-', owl_global_get_cols(&g));
+  }
 
   if (owl_global_is_sepbar_disable(&g)) {
     getyx(sepwin, y, x);
@@ -42,11 +46,11 @@ void sepbar(char *in)
   waddstr(sepwin, buff);
 
   foo=owl_view_get_filtname(v);
-  if (strcmp(foo, "all")) wattroff(sepwin, A_REVERSE);
+  if (strcmp(foo, owl_global_get_view_home(&g))) wattroff(sepwin, A_REVERSE);
   waddstr(sepwin, " ");
   waddstr(sepwin, owl_view_get_filtname(v));
   waddstr(sepwin, " ");
-  if (strcmp(foo, "all")) wattron(sepwin, A_REVERSE);
+  if (strcmp(foo, owl_global_get_view_home(&g))) wattron(sepwin, A_REVERSE);
 
   if (owl_mainwin_is_curmsg_truncated(owl_global_get_mainwin(&g))) {
     getyx(sepwin, y, x);
@@ -73,12 +77,18 @@ void sepbar(char *in)
     waddstr(sepwin, buff);
   }
 
-  if (owl_global_is_zaway(&g)) {
+  if (owl_global_is_zaway(&g) || owl_global_is_aaway(&g)) {
     getyx(sepwin, y, x);
     wmove(sepwin, y, x+2);
     wattron(sepwin, A_BOLD);
     wattroff(sepwin, A_REVERSE);
-    waddstr(sepwin, " ZAWAY ");
+    if (owl_global_is_zaway(&g) && owl_global_is_aaway(&g)) {
+      waddstr(sepwin, " AWAY ");
+    } else if (owl_global_is_zaway(&g)) {
+      waddstr(sepwin, " Z-AWAY ");
+    } else if (owl_global_is_aaway(&g)) {
+      waddstr(sepwin, " A-AWAY ");
+    }
     wattron(sepwin, A_REVERSE);
     wattroff(sepwin, A_BOLD);
   }
@@ -173,6 +183,72 @@ char *skiptokens(char *buff, int n) {
   return buff;
 }
 
+/* Return a "nice" version of the path.  Tilde expansion is done, and
+ * duplicate slashes are removed.  Caller must free the return.
+ */
+char *owl_util_makepath(char *in)
+{
+  int i, j, x;
+  char *out, user[MAXPATHLEN];
+  struct passwd *pw;
+
+  out=owl_malloc(MAXPATHLEN+1);
+  out[0]='\0';
+  j=strlen(in);
+  x=0;
+  for (i=0; i<j; i++) {
+    if (in[i]=='~') {
+      if ( (i==(j-1)) ||          /* last character */
+	   (in[i+1]=='/') ) {     /* ~/ */
+	/* use my homedir */
+	pw=getpwuid(getuid());
+	if (!pw) {
+	  out[x]=in[i];
+	} else {
+	  out[x]='\0';
+	  strcat(out, pw->pw_dir);
+	  x+=strlen(pw->pw_dir);
+	}
+      } else {
+	/* another user homedir */
+	int a, b;
+	b=0;
+	for (a=i+1; i<j; a++) {
+	  if (in[a]==' ' || in[a]=='/') {
+	    break;
+	  } else {
+	    user[b]=in[a];
+	    i++;
+	    b++;
+	  }
+	}
+	user[b]='\0';
+	pw=getpwnam(user);
+	if (!pw) {
+	  out[x]=in[i];
+	} else {
+	  out[x]='\0';
+	  strcat(out, pw->pw_dir);
+	  x+=strlen(pw->pw_dir);
+	}
+      }
+    } else if (in[i]=='/') {
+      /* check for a double / */
+      if (i<(j-1) && (in[i+1]=='/')) {
+	/* do nothing */
+      } else {
+	out[x]=in[i];
+	x++;
+      }
+    } else {
+      out[x]=in[i];
+      x++;
+    }
+  }
+  out[x]='\0';
+  return(out);
+}
+
 void atokenize_free(char **tok, int nels)
 {
   int i;
@@ -301,7 +377,6 @@ char *owl_util_minutes_to_timestr(int in)
   return(out);
 }
 
-
 /* return the index of the last char before a change from the first one */
 int owl_util_find_trans(char *in, int len)
 {
@@ -312,7 +387,6 @@ int owl_util_find_trans(char *in, int len)
   return(i);
 }
 
-
 /* downcase the string 'foo' */
 void downstr(char *foo)
 {
@@ -322,27 +396,6 @@ void downstr(char *foo)
   }
 }
 
-/* exactly like strstr but case insensitive */
-char *stristr(char *a, char *b)
-{
-  char *x, *y, *ret;
-
-  if ((x=owl_strdup(a))==NULL) return(NULL);
-  if ((y=owl_strdup(b))==NULL) return(NULL);
-  downstr(x);
-  downstr(y);
-  ret=strstr(x, y);
-  if (ret==NULL) {
-    owl_free(x);
-    owl_free(y);
-    return(NULL);
-  }
-  ret=ret-x+a;
-  owl_free(x);
-  owl_free(y);
-  return(ret);
-}
-
 /* Caller must free response. 
  * Takes in strings which are space-separated lists of tokens
  * and returns a single string containing no token more than once.
@@ -377,16 +430,6 @@ char *owl_util_uniq(char *A, char *B, char *prohibit)
   return(cat);
 }
 
-/* return 1 if a string is only whitespace, otherwise 0 */
-int only_whitespace(char *s)
-{
-  int i;
-  for (i=0; s[i]; i++) {
-    if (!isspace((int) s[i])) return(0);
-  }
-  return(1);
-}
-
 /* hooks for doing memory allocation et. al. in owl */
 
 void *owl_malloc(size_t size)
@@ -409,7 +452,6 @@ void *owl_realloc(void *ptr, size_t size)
   return(realloc(ptr, size));
 }
 
-
 /* allocates memory and returns the string or null.
  * caller must free the string. 
  * from Linux sprintf man page. 
@@ -438,140 +480,9 @@ char *owl_sprintf(const char *fmt, ...)
   }
 }
 
-/* Strip a local realm fron the zephyr user name.
- * The caller must free the return
+/* Return the owl color associated with the named color.  Return -1
+ * if the named color is not available
  */
-char *short_zuser(char *in)
-{
-  char *out, *ptr;
-
-  out=owl_strdup(in);
-  ptr=strchr(out, '@');
-  if (ptr) {
-    if (!strcasecmp(ptr+1, owl_zephyr_get_realm())) {
-      *ptr='\0';
-    }
-  }
-  return(out);
-}
-
-/* Append a local realm to the zephyr user name if necessary.
- * The caller must free the return.
- */
-char *long_zuser(char *in)
-{
-  char *ptr;
-
-  if (NULL != (ptr=strchr(in, '@'))) {
-    return owl_strdup(in);
-  } else {
-    return owl_sprintf("%s@%s", in, owl_zephyr_get_realm());
-  }
-}
-
-
-/* strip out the instance from a zsender's principal.  Preserves the
- * realm if present.  daemon.webzephyr is a special case.  The
- * caller must free the return
- */
-char *owl_util_smartstripped_user(char *in)
-{
-  char *ptr, *realm, *out;
-
-  out=owl_strdup(in);
-
-  /* bail immeaditly if we don't have to do any work */
-  ptr=strchr(in, '.');
-  if (!strchr(in, '/') && !ptr) {
-    /* no '/' and no '.' */
-    return(out);
-  }
-  if (ptr && strchr(in, '@') && (ptr > strchr(in, '@'))) {
-    /* There's a '.' but it's in the realm */
-    return(out);
-  }
-  if (!strncasecmp(in, OWL_WEBZEPHYR_PRINCIPAL, strlen(OWL_WEBZEPHYR_PRINCIPAL))) {
-    return(out);
-  }
-
-  /* remove the realm from ptr, but hold on to it */
-  realm=strchr(out, '@');
-  if (realm) realm[0]='\0';
-
-  /* strip */
-  ptr=strchr(out, '.');
-  if (!ptr) ptr=strchr(out, '/');
-  ptr[0]='\0';
-
-  /* reattach the realm if we had one */
-  if (realm) {
-    strcat(out, "@");
-    strcat(out, realm+1);
-  }
-
-  return(out);
-}
-
-char *owl_getquoting(char *line)
-{
-  if (line[0]=='\0') return("'");
-  if (strchr(line, '\'')) return("\"");
-  if (strchr(line, '"')) return("'");
-  if (strchr(line, ' ')) return("'");
-  return("");
-}
-
-
-
-/* Return a string with any occurances of 'from' replaced with 'to'.
- * Does not currently handle backslash quoting, but may in the future.
- * Caller must free returned string.
- */
-char *owl_util_substitute(char *in, char *from, char *to)
-{
-  
-  char *out;
-  int   outlen, tolen, fromlen, inpos=0, outpos=0;
-
-  if (!*from) return owl_strdup(in);
-
-  outlen = strlen(in)+1;
-  tolen  = strlen(to);
-  fromlen  = strlen(from);
-  out = malloc(outlen);
-
-  while (in[inpos]) {
-    if (!strncmp(in+inpos, from, fromlen)) {
-      outlen += tolen;
-      out = owl_realloc(out, outlen);
-      strcpy(out+outpos, to);
-      inpos += fromlen;
-      outpos += tolen;
-    } else {
-      out[outpos] = in[inpos];
-      inpos++; outpos++;
-    }
-  }
-  out[outpos] = '\0';
-  return(out);
-}
-
-/* replace all instances of character a in buff with the character
- * b.  buff must be null terminated.
- */
-void owl_util_tr(char *buff, char a, char b)
-{
-  int i;
-
-  owl_function_debugmsg("In: %s", buff);
-  for (i=0; buff[i]!='\0'; i++) {
-    if (buff[i]==a) buff[i]=b;
-  }
-  owl_function_debugmsg("Out: %s", buff);
-}
-
-
-/* Return the owl color associated with the named color */
 int owl_util_string_to_color(char *color)
 {
   if (!strcasecmp(color, "black")) {
@@ -593,7 +504,7 @@ int owl_util_string_to_color(char *color)
   } else if (!strcasecmp(color, "default")) {
     return(OWL_COLOR_DEFAULT);
   }
-  return(OWL_COLOR_DEFAULT);
+  return(-1);
 }
 
 /* Return a string name of the given owl color */
@@ -699,7 +610,7 @@ char *owl_util_stripnewlines(char *in)
     ptr1++;
   }
   ptr2=ptr1+strlen(ptr1)-1;
-  while (ptr2[0]=='\n' && ptr2>ptr1) {
+  while (ptr2>ptr1 && ptr2[0]=='\n') {
     ptr2[0]='\0';
     ptr2--;
   }
@@ -725,7 +636,7 @@ void owl_util_file_deleteline(char *filename, char *line, int backup)
   /* open the file for reading */
   file=fopen(filename, "r");
   if (!file) {
-    owl_function_makemsg("Error opening file %s", filename);
+    owl_function_error("Error opening file %s", filename);
     return;
   }
 
@@ -734,7 +645,7 @@ void owl_util_file_deleteline(char *filename, char *line, int backup)
     backupfilename=owl_sprintf("%s.backup", filename);
     backupfile=fopen(backupfilename, "w");
     if (!backupfile) {
-      owl_function_makemsg("Error opening file %s for writing", backupfilename);
+      owl_function_error("Error opening file %s for writing", backupfilename);
       owl_free(backupfilename);
       return;
     }
@@ -764,7 +675,10 @@ void owl_util_file_deleteline(char *filename, char *line, int backup)
     }
 
     /* write to backupfile if necessary */
-    if (backup) fputs(buff, backupfile);
+    if (backup) {
+      fputs(buff, backupfile);
+      if (newline) fputs("\n", backupfile);
+    }
   }
   if (backup) fclose(backupfile);
   fclose(file);
@@ -772,7 +686,7 @@ void owl_util_file_deleteline(char *filename, char *line, int backup)
   /* now rewrite the original file from memory */
   file=fopen(filename, "w");
   if (!file) {
-    owl_function_makemsg("WARNING: Error opening %s for writing.  Use %s to restore.", filename, backupfilename);
+    owl_function_error("WARNING: Error opening %s for writing.  Use %s to restore.", filename, backupfilename);
     owl_function_beep();
     owl_free(line);
     return;
@@ -782,6 +696,46 @@ void owl_util_file_deleteline(char *filename, char *line, int backup)
   fclose(file);
 }
 
+/* add the string 'str' to the list 'list' of strings, only if it
+ * is not already present
+ */
+void owl_util_list_add_unique_string(owl_list *list, char *str)
+{
+  int i, j;
+
+  j=owl_list_get_size(list);
+  for (i=0; i<j; i++) {
+    if (!strcmp(str, owl_list_get_element(list, i))) return;
+  }
+  owl_list_append_element(list, owl_strdup(str));
+}
+
+int owl_util_common_strings_in_lists(owl_list *a, owl_list *b)
+{
+  int i, j, x, y;
+
+  j=owl_list_get_size(a);
+  for (i=0; i<j; i++) {
+    y=owl_list_get_size(b);
+    for (x=0; x<y; x++) {
+      if (!strcmp(owl_list_get_element(a, i), owl_list_get_element(b, x))) return(1);
+    }
+  }
+  return(0);
+}
+
+int owl_util_max(int a, int b)
+{
+  if (a>b) return(a);
+  return(b);
+}
+
+int owl_util_min(int a, int b)
+{
+  if (a<b) return(a);
+  return(b);
+}
+
 /**************************************************************************/
 /************************* REGRESSION TESTS *******************************/
 /**************************************************************************/
@@ -797,13 +751,13 @@ int owl_util_regtest(void)
   printf("BEGIN testing owl_util\n");
 
   FAIL_UNLESS("owl_util_substitute 1",
-	      !strcmp("foo", owl_util_substitute("foo", "", "Y")));
-  FAIL_UNLESS("owl_util_substitute 2",
-	      !strcmp("fYZYZ", owl_util_substitute("foo", "o", "YZ")));
-  FAIL_UNLESS("owl_util_substitute 3",
-	      !strcmp("foo", owl_util_substitute("fYZYZ", "YZ", "o")));
-  FAIL_UNLESS("owl_util_substitute 4",
-	      !strcmp("/u/foo/meep", owl_util_substitute("~/meep", "~", "/u/foo")));
+	      !strcmp("foo", owl_text_substitute("foo", "", "Y")));
+  FAIL_UNLESS("owl_text_substitute 2",
+	      !strcmp("fYZYZ", owl_text_substitute("foo", "o", "YZ")));
+  FAIL_UNLESS("owl_text_substitute 3",
+	      !strcmp("foo", owl_text_substitute("fYZYZ", "YZ", "o")));
+  FAIL_UNLESS("owl_text_substitute 4",
+	      !strcmp("/u/foo/meep", owl_text_substitute("~/meep", "~", "/u/foo")));
 
   FAIL_UNLESS("skiptokens 1", 
 	      !strcmp("bar quux", skiptokens("foo bar quux", 1)));
