@@ -5,7 +5,7 @@
 #include <ctype.h>
 #include "owl.h"
 
-static const char fileIdent[] = "$Id: variable.c,v 1.27 2003/07/06 22:42:35 nygren Exp $";
+static const char fileIdent[] = "$Id: variable.c,v 1.40 2004/02/23 07:04:25 kretch Exp $";
 
 static int in_regtest = 0;
 
@@ -56,9 +56,10 @@ static int in_regtest = 0;
 
 static owl_variable variables_to_init[] = {
 
-  OWLVAR_BOOL( "personalbell" /* %OwlVarStub */, 0,
-	       "ring the terminal bell when personal messages are received",
-	       "" ),
+  OWLVAR_STRING( "personalbell" /* %OwlVarStub */, "off",
+		 "ring the terminal bell when personal messages are received",
+		 "Can be set to 'on', 'off', or the name of a filter which\n"
+		 "messages need to match in order to ring the bell"),
 
   OWLVAR_BOOL( "bell" /* %OwlVarStub */, 1,
 	       "enable / disable the terminal bell", "" ),
@@ -111,15 +112,50 @@ static owl_variable variables_to_init[] = {
 	       "The filename in that directory is derived from\n"
 	       "the name of the class to which the message was sent.\n" ),
 
+  OWLVAR_ENUM( "loggingdirection" /* %OwlVarStub */, OWL_LOGGING_DIRECTION_BOTH,
+	       "specifices which kind of messages should be logged",
+	       "Can be one of 'both', 'in', or 'out'.  If 'in' is\n"
+	       "selected, only incoming messages are logged, if 'out'\n"
+	       "is selected only outgoing messages are logged.  If 'both'\n"
+	       "is selected both incoming and outgoing messages are\n"
+	       "logged.",
+	       "both,in,out"),
+
   OWLVAR_BOOL( "colorztext" /* %OwlVarStub */, 1,
 	       "allow @color() in zephyrs to change color",
 	       "Note that only messages received after this variable\n"
 	       "is set will be affected." ),
 
+  OWLVAR_BOOL( "fancylines" /* %OwlVarStub */, 1,
+	       "Use 'nice' line drawing on the terminal.",
+	       "If turned off, dashes, pipes and pluses will be used\n"
+	       "to draw lines on the screen.  Useful when the terminal\n"
+	       "is causing problems" ),
+
   OWLVAR_BOOL( "zcrypt" /* %OwlVarStub */, 1,
 	       "Do automatic zcrypt processing",
 	       "" ),
 
+  OWLVAR_BOOL_FULL( "pseudologins" /* %OwlVarStub */, 0,
+		    "Enable zephyr pseudo logins",
+		    "When this is enabled, Owl will periodically check the zephyr\n"
+		    "location of users in your .anyone file.  If a user is present\n"
+		    "but sent no login message, or a user is not present that sent no\n"
+		    "logout message a pseudo login or logout message wil be created\n",
+		    NULL, owl_variable_pseudologins_set, NULL),
+
+  OWLVAR_BOOL( "ignorelogins" /* %OwlVarStub */, 0,
+	       "Enable printing of login notifications",
+	       "When this is enabled, Owl will print login and logout notifications\n"
+	       "for AIM, zephyr, or other protocols.  If disabled Owl will not print\n"
+	       "login or logout notifications.\n"),
+
+  OWLVAR_BOOL( "loglogins" /* %OwlVarStub */, 0,
+	       "Enable logging of login notifications",
+	       "When this is enabled, Owl will login login and logout notifications\n"
+	       "for AIM, zephyr, or other protocols.  If disabled Owl will not print\n"
+	       "login or logout notifications.\n"),
+
   OWLVAR_ENUM_FULL( "disable-ctrl-d" /* %OwlVarStub:lockout_ctrld */, 1,
 		    "don't send zephyrs on C-d",
 		    "If set to 'off', C-d won't send a zephyr from the edit\n"
@@ -186,6 +222,19 @@ static owl_variable variables_to_init[] = {
 		 OWL_DEFAULT_ZAWAYMSG,
 	         "default zaway message", "" ),
 
+  OWLVAR_BOOL_FULL( "aaway" /* %OwlVarStub */, 0,
+		    "Set AIM away status",
+		    "",
+		    NULL, owl_variable_aaway_set, NULL),
+
+  OWLVAR_STRING( "aaway_msg" /* %OwlVarStub */, 
+		 OWL_DEFAULT_AAWAYMSG,
+	         "AIM away msg for responding when away", "" ),
+
+  OWLVAR_STRING( "aaway_msg_default" /* %OwlVarStub */, 
+		 OWL_DEFAULT_AAWAYMSG,
+	         "default AIM away message", "" ),
+
   OWLVAR_STRING( "view_home" /* %OwlVarStub */, "all",
 	         "home view to switch to after 'X' and 'V'", 
 		 "SEE ALSO: view, filter\n" ),
@@ -201,7 +250,7 @@ static owl_variable variables_to_init[] = {
   OWLVAR_STRING_FULL( "tty" /* %OwlVarStub */, "", "tty name for zephyr location", "",
 		      NULL, owl_variable_tty_set, NULL),
 
-  OWLVAR_STRING( "default_style" /* %OwlVarStub */, "default",
+  OWLVAR_STRING( "default_style" /* %OwlVarStub */, "__unspecified__",
 		 "name of the default formatting style",
 		 "This sets the default message formatting style.\n"
 		 "Styles may be created with the 'style' command.\n"
@@ -234,13 +283,12 @@ static owl_variable variables_to_init[] = {
 		 "It is recommended that outgoing messages be no wider\n"
 		 "than 60 columns, as a courtesy to recipients.\n"),
 
-  OWLVAR_INT( "aim_ignorelogin_timer" /* %OwlVarStub */, 0,
+  OWLVAR_INT( "aim_ignorelogin_timer" /* %OwlVarStub */, 15,
 	      "number of seconds after AIM login to ignore login messages",
 	      "This specifies the number of seconds to wait after an\n"
 	      "AIM login before allowing the recipt of AIM login notifications.\n"
-	      "By default this is set to 0 and all login notifications are\n"
-	      "viewed immediately.  If you dislike this behavior try setting\n"
-	      "the value to 20 instead.\n"),
+	      "By default this is set to 15.  If you would like to view login\n"
+	      "notifications of buddies as soon as you login, set it to 0 instead."),
 
 	      
   OWLVAR_INT_FULL( "typewinsize" /* %OwlVarStub:typwin_lines */, 
@@ -311,21 +359,23 @@ static owl_variable variables_to_init[] = {
 
 /* commonly useful */
 
-int owl_variable_int_validate_gt0(owl_variable *v, void *newval) {
+int owl_variable_int_validate_gt0(owl_variable *v, void *newval)
+{
   if (newval == NULL) return(0);
   else if (*(int*)newval < 1) return(0);
   else return (1);
 }
 
-int owl_variable_int_validate_positive(owl_variable *v, void *newval) {
+int owl_variable_int_validate_positive(owl_variable *v, void *newval)
+{
   if (newval == NULL) return(0);
   else if (*(int*)newval < 0) return(0);
   else return (1);
 }
 
 /* typewinsize */
-
-int owl_variable_typewinsize_set(owl_variable *v, void *newval) {
+int owl_variable_typewinsize_set(owl_variable *v, void *newval)
+{
   int rv;
   rv = owl_variable_int_set_default(v, newval);
   if (0 == rv) owl_function_resize();
@@ -333,17 +383,41 @@ int owl_variable_typewinsize_set(owl_variable *v, void *newval) {
 }
 
 /* debug (cache value in g->debug) */
-
-int owl_variable_debug_set(owl_variable *v, void *newval) {
+int owl_variable_debug_set(owl_variable *v, void *newval)
+{
   if (newval && (*(int*)newval == 1 || *(int*)newval == 0)) {
     g.debug = *(int*)newval;
   }
   return owl_variable_bool_set_default(v, newval);
 }
 
+/* When 'aaway' is changed, need to notify the AIM server */
+int owl_variable_aaway_set(owl_variable *v, void *newval)
+{
+  if (newval) {
+    if (*(int*)newval == 1) {
+      owl_aim_set_awaymsg(owl_global_get_aaway_msg(&g));
+    } else if (*(int*)newval == 0) {
+      owl_aim_set_awaymsg("");
+    }
+  }
+  return owl_variable_bool_set_default(v, newval);
+}
+
+int owl_variable_pseudologins_set(owl_variable *v, void *newval)
+{
+  if (newval) {
+    if (*(int*)newval == 1) {
+      owl_function_zephyr_buddy_check(0);
+    }
+  }
+  return owl_variable_bool_set_default(v, newval);
+}
+
 /* note that changing the value of this will clobber 
  * any user setting of this */
-int owl_variable_disable_ctrl_d_set(owl_variable *v, void *newval) {
+int owl_variable_disable_ctrl_d_set(owl_variable *v, void *newval)
+{
 
   if (in_regtest) return owl_variable_int_set_default(v, newval);
 
@@ -359,7 +433,8 @@ int owl_variable_disable_ctrl_d_set(owl_variable *v, void *newval) {
   return owl_variable_int_set_default(v, newval);  
 }
 
-int owl_variable_tty_set(owl_variable *v, void *newval) {
+int owl_variable_tty_set(owl_variable *v, void *newval)
+{
   owl_zephyr_set_locationinfo(owl_global_get_hostname(&g), newval);
   return(owl_variable_string_set_default(v, newval));
 }
@@ -476,19 +551,19 @@ int owl_variable_set_fromstring(owl_vardict *d, char *name, char *value, int msg
   if (!name) return(-1);
   v = owl_dict_find_element(d, name);
   if (v == NULL) {
-    if (msg) owl_function_makemsg("Unknown variable %s", name);
+    if (msg) owl_function_error("Unknown variable %s", name);
     return -1;
   }
   if (!v->set_fromstring_fn) {
-    if (msg) owl_function_makemsg("Variable %s is read-only", name);
+    if (msg) owl_function_error("Variable %s is read-only", name);
     return -1;   
   }
   if (requirebool && v->type!=OWL_VARIABLE_BOOL) {
-    if (msg) owl_function_makemsg("Variable %s is not a boolean", name);
+    if (msg) owl_function_error("Variable %s is not a boolean", name);
     return -1;   
   }
   if (0 != v->set_fromstring_fn(v, value)) {
-    if (msg) owl_function_makemsg("Unable to set %s (must be %s)", name, 
+    if (msg) owl_function_error("Unable to set %s (must be %s)", name, 
 				  owl_variable_get_validsettings(v));
     return -1;
   }
