@@ -2,10 +2,11 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <sys/stat.h>
 #include <string.h>
 #include "owl.h"
 
-static const char fileIdent[] = "$Id: zephyr.c,v 1.24 2003/07/06 22:42:38 nygren Exp $";
+static const char fileIdent[] = "$Id: zephyr.c,v 1.47 2004/04/18 15:55:42 kretch Exp $";
 
 #ifdef HAVE_LIBZEPHYR
 Code_t ZResetAuthentication();
@@ -65,27 +66,31 @@ char *owl_zephyr_get_sender()
 #endif
 }
 
+/* return 0  on success
+ *        -1 on file error
+ *        -2 on subscription error
+ */
 int owl_zephyr_loadsubs(char *filename)
 {
 #ifdef HAVE_LIBZEPHYR
-  /* return 0  on success
-   *        -1 on file error
-   *        -2 on subscription error
-   */
   FILE *file;
   char *tmp, *start;
   char buffer[1024], subsfile[1024];
   ZSubscription_t subs[3001];
   int count, ret, i;
+  struct stat statbuff;
 
-  ret=0;
-  
   if (filename==NULL) {
     sprintf(subsfile, "%s/%s", owl_global_get_homedir(&g), ".zephyr.subs");
   } else {
     strcpy(subsfile, filename);
   }
 
+  ret=stat(subsfile, &statbuff);
+  if (ret) return(0);
+
+  ret=0;
+
   ZResetAuthentication();
   /* need to redo this to do chunks, not just bail after 3000 */
   count=0;
@@ -123,9 +128,9 @@ int owl_zephyr_loadsubs(char *filename)
     ret=-1;
   }
 
-  /* sub with defaults */
-  if (ZSubscribeTo(subs,count,0) != ZERR_NONE) {
-    fprintf(stderr, "Error subbing\n");
+  /* sub without defaults */
+  if (ZSubscribeToSansDefaults(subs,count,0) != ZERR_NONE) {
+    owl_function_error("Error subscribing to zephyr notifications.");
     ret=-2;
   }
 
@@ -142,6 +147,21 @@ int owl_zephyr_loadsubs(char *filename)
 #endif
 }
 
+int owl_zephyr_loaddefaultsubs()
+{
+#ifdef HAVE_LIBZEPHYR
+  ZSubscription_t subs[10];
+    
+  if (ZSubscribeTo(subs,0,0) != ZERR_NONE) {
+    owl_function_error("Error subscribing to default zephyr notifications.");
+    return(-1);
+  }
+  return(0);
+#else
+  return(0);
+#endif
+}
+
 int owl_zephyr_loadloginsubs(char *filename)
 {
 #ifdef HAVE_LIBZEPHYR
@@ -149,14 +169,18 @@ int owl_zephyr_loadloginsubs(char *filename)
   ZSubscription_t subs[3001];
   char subsfile[1024], buffer[1024];
   int count, ret, i;
-
-  ret=0;
+  struct stat statbuff;
 
   if (filename==NULL) {
     sprintf(subsfile, "%s/%s", owl_global_get_homedir(&g), ".anyone");
   } else {
     strcpy(subsfile, filename);
   }
+  
+  ret=stat(subsfile, &statbuff);
+  if (ret) return(0);
+
+  ret=0;
 
   ZResetAuthentication();
   /* need to redo this to do chunks, not just bag out after 3000 */
@@ -187,7 +211,7 @@ int owl_zephyr_loadloginsubs(char *filename)
 
   /* sub with defaults */
   if (ZSubscribeToSansDefaults(subs,count,0) != ZERR_NONE) {
-    fprintf(stderr, "Error subbing\n");
+    owl_function_error("Error subscribing to zephyr notifications.");
     ret=-2;
   }
 
@@ -219,7 +243,6 @@ int owl_zephyr_sub(char *class, char *inst, char *recip)
 {
 #ifdef HAVE_LIBZEPHYR
   ZSubscription_t subs[5];
-  int ret;
 
   subs[0].zsub_class=class;
   subs[0].zsub_classinst=inst;
@@ -227,8 +250,8 @@ int owl_zephyr_sub(char *class, char *inst, char *recip)
 
   ZResetAuthentication();
   if (ZSubscribeTo(subs,1,0) != ZERR_NONE) {
-    fprintf(stderr, "Error subbing\n");
-    ret=-2;
+    owl_function_error("Error subbing to <%s,%s,%s>", class, inst, recip);
+    return(-2);
   }
   return(0);
 #else
@@ -241,7 +264,6 @@ int owl_zephyr_unsub(char *class, char *inst, char *recip)
 {
 #ifdef HAVE_LIBZEPHYR
   ZSubscription_t subs[5];
-  int ret;
 
   subs[0].zsub_class=class;
   subs[0].zsub_classinst=inst;
@@ -249,8 +271,8 @@ int owl_zephyr_unsub(char *class, char *inst, char *recip)
 
   ZResetAuthentication();
   if (ZUnsubscribeTo(subs,1,0) != ZERR_NONE) {
-    fprintf(stderr, "Error unsubbing\n");
-    ret=-2;
+    owl_function_error("Error unsubbing from <%s,%s,%s>", class, inst, recip);
+    return(-2);
   }
   return(0);
 #else
@@ -258,12 +280,14 @@ int owl_zephyr_unsub(char *class, char *inst, char *recip)
 #endif
 }
 
+/* return a pointer to the data in the Jth field, (NULL terminated by
+ * definition).  Caller must free the return.
+ */
 #ifdef HAVE_LIBZEPHYR
-char *owl_zephyr_get_field(ZNotice_t *n, int j, int *k)
+char *owl_zephyr_get_field(ZNotice_t *n, int j)
 {
-  /* return a pointer to the Jth field, place the length in k.  If the
-     field doesn't exist return an emtpy string */
   int i, count, save;
+  char *out;
 
   count=save=0;
   for (i=0; i<n->z_message_len; i++) {
@@ -271,29 +295,30 @@ char *owl_zephyr_get_field(ZNotice_t *n, int j, int *k)
       count++;
       if (count==j) {
 	/* just found the end of the field we're looking for */
-	*k=i-save;
-	return(n->z_message+save);
+	return(owl_strdup(n->z_message+save));
       } else {
 	save=i+1;
       }
     }
   }
-  /* catch the last field */
+  /* catch the last field, which might not be null terminated */
   if (count==j-1) {
-    *k=n->z_message_len-save;
-    return(n->z_message+save);
+    out=owl_malloc(n->z_message_len-save+5);
+    memcpy(out, n->z_message+save, n->z_message_len-save);
+    out[n->z_message_len-save]='\0';
+    return(out);
   }
-  
-  *k=0;
-  return("");
+
+  return(owl_strdup(""));
 }
 #else
-char *owl_zephyr_get_field(void *n, int j, int *k)
+char *owl_zephyr_get_field(void *n, int j)
 {
-  return("");
+  return(owl_strdup(""));
 }
 #endif
 
+
 #ifdef HAVE_LIBZEPHYR
 int owl_zephyr_get_num_fields(ZNotice_t *n)
 {
@@ -314,15 +339,22 @@ int owl_zephyr_get_num_fields(void *n)
 #endif
 
 #ifdef HAVE_LIBZEPHYR
-char *owl_zephyr_get_message(ZNotice_t *n, int *k)
+/* return a pointer to the message, place the message length in k
+ * caller must free the return
+ */
+char *owl_zephyr_get_message(ZNotice_t *n)
 {
-  /* return a pointer to the message, place the message length in k */
+  /* don't let ping messages have a body */
   if (!strcasecmp(n->z_opcode, "ping")) {
-    *k=0;
-    return("");
+    return(owl_strdup(""));
   }
 
-  return(owl_zephyr_get_field(n, 2, k));
+  /* deal with MIT Athena OLC messages */
+  if (!strcasecmp(n->z_sender, "olc.matisse@ATHENA.MIT.EDU")) {
+    return(owl_zephyr_get_field(n, 1));
+  }
+
+  return(owl_zephyr_get_field(n, 2));
 }
 #endif
 
@@ -331,10 +363,18 @@ char *owl_zephyr_get_zsig(ZNotice_t *n, int *k)
 {
   /* return a pointer to the zsig if there is one */
 
+  /* message length 0? No zsig */
   if (n->z_message_len==0) {
     *k=0;
     return("");
   }
+
+  /* No zsig for OLC messages */
+  if (!strcasecmp(n->z_sender, "olc.matisse@ATHENA.MIT.EDU")) {
+    return("");
+  }
+
+  /* Everything else is field 1 */
   *k=strlen(n->z_message);
   return(n->z_message);
 }
@@ -354,6 +394,8 @@ int send_zephyr(char *opcode, char *zsig, char *class, char *instance, char *rec
   memset(&notice, 0, sizeof(notice));
 
   ZResetAuthentication();
+
+  if (!zsig) zsig="";
   
   notice.z_kind=ACKED;
   notice.z_port=0;
@@ -380,7 +422,7 @@ int send_zephyr(char *opcode, char *zsig, char *class, char *instance, char *rec
   owl_free(notice.z_message);
   ZFreeNotice(&notice);
   if (ret!=ZERR_NONE) {
-    owl_function_makemsg("Error sending zephyr");
+    owl_function_error("Error sending zephyr");
     return(ret);
   }
   return(0);
@@ -412,9 +454,9 @@ void owl_zephyr_handle_ack(ZNotice_t *retnotice)
   if (retnotice->z_kind == HMACK) return;
 
   if (retnotice->z_kind == SERVNAK) {
-    owl_function_makemsg("Authorization failure sending zephyr");
+    owl_function_error("Authorization failure sending zephyr");
   } else if ((retnotice->z_kind != SERVACK) || !retnotice->z_message_len) {
-    owl_function_makemsg("Detected server failure while receiving acknowledgement");
+    owl_function_error("Detected server failure while receiving acknowledgement");
   } else if (!strcmp(retnotice->z_message, ZSRVACK_SENT)) {
     if (!strcasecmp(retnotice->z_opcode, "ping")) {
       return;
@@ -428,16 +470,23 @@ void owl_zephyr_handle_ack(ZNotice_t *retnotice)
     }
   } else if (!strcmp(retnotice->z_message, ZSRVACK_NOTSENT)) {
     if (strcasecmp(retnotice->z_class, "message")) {
-      owl_function_makemsg("Not logged in or not subscribing to class %s, instance %s",
-			   retnotice->z_class, retnotice->z_class_inst);
+      char buff[1024];
+      owl_function_error("No one subscribed to class class %s", retnotice->z_class);
+      sprintf(buff, "Could not send message to class %s: no one subscribed.\n", retnotice->z_class);
+      owl_function_adminmsg("", buff);
     } else {
+      char buff[1024];
       tmp = short_zuser(retnotice->z_recipient);
-      owl_function_makemsg("%s: Not logged in or subscribing to messages.", 
+      owl_function_error("%s: Not logged in or subscribing to messages.", 
 			   tmp);
+
+      sprintf(buff, "Could not send message to %s: not logged in or subscribing to messages.\n", tmp);
+      owl_function_adminmsg("", buff);
+      if (owl_global_is_logging(&g)) owl_log_outgoing_zephyr_error(tmp, buff);
       owl_free(tmp);
     }
   } else {
-    owl_function_makemsg("Internal error on ack (%s)", retnotice->z_message);
+    owl_function_error("Internal error on ack (%s)", retnotice->z_message);
   }
 }
 #else
@@ -468,7 +517,8 @@ void owl_zephyr_zaway(owl_message *m)
   char *tmpbuff, *myuser, *to;
   
   /* bail if it doesn't look like a message we should reply to.  Some
-     of this defined by the way zaway(1) works */
+   * of this defined by the way zaway(1) works
+   */
   if (strcasecmp(owl_message_get_class(m), "message")) return;
   if (strcasecmp(owl_message_get_recipient(m), ZGetSender())) return;
   if (!strcasecmp(owl_message_get_sender(m), "")) return;
@@ -476,9 +526,10 @@ void owl_zephyr_zaway(owl_message *m)
   if (!strcasecmp(owl_message_get_opcode(m), "auto")) return;
   if (!strcasecmp(owl_message_get_zsig(m), "Automated reply:")) return;
   if (!strcasecmp(owl_message_get_sender(m), ZGetSender())) return;
+  if (owl_message_get_attribute_value(m, "isauto")) return;
 
   if (owl_global_is_smartstrip(&g)) {
-    to=owl_util_smartstripped_user(owl_message_get_sender(m));
+    to=owl_zephyr_smartstripped_user(owl_message_get_sender(m));
   } else {
     to=owl_strdup(owl_message_get_sender(m));
   }
@@ -531,7 +582,7 @@ void owl_zephyr_zlocate(char *user, char *out, int auth)
   ZResetAuthentication();
   ret=ZLocateUser(user,&numlocs,auth?ZAUTH:ZNOAUTH);
   if (ret != ZERR_NONE) {
-    owl_function_makemsg("Error locating user");
+    owl_function_error("Error locating user %s", user);
   }
 
   if (numlocs==0) {
@@ -567,13 +618,13 @@ void owl_zephyr_addsub(char *filename, char *class, char *inst, char *recip)
   /* first check if it exists already */
   file=fopen(subsfile, "r");
   if (!file) {
-    owl_function_makemsg("Error opening file %s", subsfile);
+    owl_function_error("Error opening file %s", subsfile);
     owl_free(line);
     return;
   }
   while (fgets(buff, LINE, file)!=NULL) {
     if (!strcasecmp(buff, line)) {
-      owl_function_makemsg("Subscription already present in %s", subsfile);
+      owl_function_error("Subscription already present in %s", subsfile);
       owl_free(line);
       return;
     }
@@ -583,7 +634,7 @@ void owl_zephyr_addsub(char *filename, char *class, char *inst, char *recip)
   fclose(file);
   file=fopen(subsfile, "a");
   if (!file) {
-    owl_function_makemsg("Error opening file %s for writing", subsfile);
+    owl_function_error("Error opening file %s for writing", subsfile);
     owl_free(line);
     return;
   }
@@ -690,7 +741,7 @@ void owl_zephyr_addbuddy(char *name)
   file=fopen(filename, "a");
   owl_free(filename);
   if (!file) {
-    owl_function_makemsg("Error opening zephyr buddy file for append");
+    owl_function_error("Error opening zephyr buddy file for append");
     return;
   }
   fprintf(file, "%s\n", name);
@@ -730,8 +781,8 @@ char *owl_zephyr_get_authstr(void *n)
 }
 #endif
 
-/* returns a buffer of subscriptions or an error message.
- * Caller must free the return.
+/* Returns a buffer of subscriptions or an error message.  Caller must
+ * free the return.
  */
 char *owl_zephyr_getsubs()
 {
@@ -743,8 +794,9 @@ char *owl_zephyr_getsubs()
 
   ret=ZRetrieveSubscriptions(0, &num);
   if (ret==ZERR_TOOMANYSUBS) {
-    out=owl_strdup("Zephyr: too many subscriptions\n");
-    return(out);
+    return(owl_strdup("Zephyr: too many subscriptions\n"));
+  } else if (ret) {
+    return(owl_strdup("Zephyr: error retriving subscriptions\n"));
   }
 
   out=owl_malloc(num*500);
@@ -767,7 +819,7 @@ char *owl_zephyr_getsubs()
   ZFlushSubscriptions();
   return(out);
 #else
-  return("");
+  return(owl_strdup("Zephyr not available"));
 #endif
 }
 
@@ -787,3 +839,129 @@ void owl_zephyr_set_locationinfo(char *host, char *val)
 #endif
 }
   
+/* Strip a local realm fron the zephyr user name.
+ * The caller must free the return
+ */
+char *short_zuser(char *in)
+{
+  char *out, *ptr;
+
+  out=owl_strdup(in);
+  ptr=strchr(out, '@');
+  if (ptr) {
+    if (!strcasecmp(ptr+1, owl_zephyr_get_realm())) {
+      *ptr='\0';
+    }
+  }
+  return(out);
+}
+
+/* Append a local realm to the zephyr user name if necessary.
+ * The caller must free the return.
+ */
+char *long_zuser(char *in)
+{
+  if (strchr(in, '@')) {
+    return(owl_strdup(in));
+  }
+  return(owl_sprintf("%s@%s", in, owl_zephyr_get_realm()));
+}
+
+/* strip out the instance from a zsender's principal.  Preserves the
+ * realm if present.  daemon.webzephyr is a special case.  The
+ * caller must free the return
+ */
+char *owl_zephyr_smartstripped_user(char *in)
+{
+  char *ptr, *realm, *out;
+
+  out=owl_strdup(in);
+
+  /* bail immeaditly if we don't have to do any work */
+  ptr=strchr(in, '.');
+  if (!strchr(in, '/') && !ptr) {
+    /* no '/' and no '.' */
+    return(out);
+  }
+  if (ptr && strchr(in, '@') && (ptr > strchr(in, '@'))) {
+    /* There's a '.' but it's in the realm */
+    return(out);
+  }
+  if (!strncasecmp(in, OWL_WEBZEPHYR_PRINCIPAL, strlen(OWL_WEBZEPHYR_PRINCIPAL))) {
+    return(out);
+  }
+
+  /* remove the realm from ptr, but hold on to it */
+  realm=strchr(out, '@');
+  if (realm) realm[0]='\0';
+
+  /* strip */
+  ptr=strchr(out, '.');
+  if (!ptr) ptr=strchr(out, '/');
+  ptr[0]='\0';
+
+  /* reattach the realm if we had one */
+  if (realm) {
+    strcat(out, "@");
+    strcat(out, realm+1);
+  }
+
+  return(out);
+}
+
+/* read the list of users in 'filename' as a .anyone file, and put the
+ * names of the zephyr users in the list 'in'.  If 'filename' is NULL,
+ * use the default .anyone file in the users home directory.  Returns
+ * -1 on failure, 0 on success.
+ */
+int owl_zephyr_get_anyone_list(owl_list *in, char *filename)
+{
+#ifdef HAVE_LIBZEPHYR
+  char *ourfile, *tmp, buff[LINE];
+  FILE *f;
+
+  if (filename==NULL) {
+    tmp=owl_global_get_homedir(&g);
+    ourfile=owl_sprintf("%s/.anyone", owl_global_get_homedir(&g));
+  } else {
+    ourfile=owl_strdup(filename);
+  }
+  
+  f=fopen(ourfile, "r");
+  if (!f) {
+    owl_function_error("Error opening file %s: %s", ourfile, strerror(errno) ? strerror(errno) : "");
+    owl_free(ourfile);
+    return(-1);
+  }
+
+  while (fgets(buff, LINE, f)!=NULL) {
+    /* ignore comments, blank lines etc. */
+    if (buff[0]=='#') continue;
+    if (buff[0]=='\n') continue;
+    if (buff[0]=='\0') continue;
+    
+    /* strip the \n */
+    buff[strlen(buff)-1]='\0';
+    
+    /* ingore from # on */
+    tmp=strchr(buff, '#');
+    if (tmp) tmp[0]='\0';
+    
+    /* ingore from SPC */
+    tmp=strchr(buff, ' ');
+    if (tmp) tmp[0]='\0';
+    
+    /* stick on the local realm. */
+    if (!strchr(buff, '@')) {
+      strcat(buff, "@");
+      strcat(buff, ZGetRealm());
+    }
+    owl_list_append_element(in, owl_strdup(buff));
+  }
+  fclose(f);
+  owl_free(ourfile);
+  return(0);
+#else
+  return(-1);
+#endif
+}
